c-----------------------------------------------------------------------
c --- TRAJ2D -- Creates 2D trajectories from a Direct-Access wind file
c-----------------------------------------------------------------------
c
c --- TRAJ2D    Version: 1.25           Level: 090407              MAIN
c
c               D. Strimaitis
c
c     Copyright (c) 2006-2009 by Exponent, Inc.
c
c --- PURPOSE: Reads direct access 1D or 2D wind files & creates
c              a series of backward trajectories
c
c --- UPDATES:
c
c --- V1.24 (090224) to V1.25 (090407)   DGS
c       (1) Redefine the EMPTY string to use ASCII code for the null
c           to avoid automatic change from null to blank by some
c           editors
c       Modified:  RDMET
c       (2) MXNX,MXNY changed from 500 to 1000
c       Modified:  PARAMS.TRJ
c       (3) COORDLIB updated to v1.99
c
c --- V1.23 (081223) to V1.24 (090224)   IWL
c       (1) Write TRJ filenames to output list file
c DGS   (2) Set valid lat/lon strings when blanks are read in MET1
c
c       Modified:  COMP, MET1
c
c --- V1.22 (081222) to V1.23 (081223)  DGS
c       (1) Swap order of lat/lon columns to LON,LAT
c       (2) Revise time format used for filename to
c           _UTC+HHMM_YYYYMMDDHHSSSS of latest start-time
c
c       Modified:  COMP, WRHDTRJ, GETLOC
c
c --- V1.21 (081124) to V1.22 (081222)  DGS
c       (1) Recognize latitude-longitude coordinates from the 
c           optional trajectory-point input file
c       (2) Change file extension of output trajectory files to .TRJ
c       (3) Apply time-shift if time zone in met file is different from
c           specified time zone for trajectory (change met times)
c       (4) Add option for coordinates used for output trajectories
c           (eventually this should always be lat/lon)
c       (5) Add full coordinate documentation to header of output file
c           using standard header structure with comment lines
c       (6) Allow for snapshot timeperiods in met file and treat
c           as constant for timeslot (period between updates)
c
c       New:       WRHDTRJ, TSHORT
c       Modified:  CTLDAT.TRJ
c                  BLOCKDATA, RDHDLOC, GETLOC, MET1, RDMET, RDCF, COMP
c
c --- V1.2 (081105) to V1.21 (081124)  DGS
c       (1) Recognize interpolated W1D records labeled 'UV-1DI'
c       Modified:  RDMET
c
c --- V1.1 (061222) to V1.2 (081105)  DGS
c       (1) Add 1D wind data for single-point met locations/models
c           The DA wind file has data for 1 cell, but still contains
c           a computational grid for the 2D trajectory domain
c       (2) Do not echo input control file to list file when control
c           information is provided in the TRAJLOC file (MFILEX=1)
c
c       Modified:  WINDHD.TRJ
c                  SETUP, MET1, RDMET, QAMET
c
c --- V1.0 (061023) to V1.1 (061222)  DGS
c       (1) Add option to read trajectory starting location and
c           time window from a TRAJLOC.DAT file (MFILEX option)
c       (2) Place name of starting location in control file
c       (3) Add limit on number of hours to trace a trajectory (NHOURS)
c       (4) Add extra header information in trajectory file for plots
c       (5) Add option to place peak concentration at start of the 
c           output trajectory filename
c       Modified:  CTLDAT.TRJ
c                  SETUP, BLOCK DATA, RDCF, COMP
c       New:       RDHDLOC, GETLOC
c
c --- TRAJ2D calls:     SETUP, COMP, FIN
c-----------------------------------------------------------------------
      Program TRAJ2D

c --- include QA common block
      include 'qa.trj'

c --- set version and level number of program
      ver='1.25'
      level='090407'

c --- Setup phase -- initialization & program setup operations
      call SETUP

c --- Computational phase -- basic time loop
      call COMP

c --- Termination phase -- program termination functions
      call FIN

      stop
      end

c----------------------------------------------------------------------
      BLOCK DATA
c----------------------------------------------------------------------
c
c --- TRAJ2D    Version: 1.25     Level: 081222              BLOCKDATA
c ---           D. Strimaitis
c
c --- UPDATES:
c --- V1.1 (061222) to V1.22 (081222)  DGS
c        - Add MCOORD
c --- V1.0 (061023) to V1.1 (061222)  DGS
c        - Add TRAJLOC.DAT and MFILEX
c        - Initialize lat/lon string variables
c        - Add max number of hours in trajectory
c
c----------------------------------------------------------------------
c --- Include parameter statements
      include 'params.trj'

c --- Include common blocks
      include 'ctldat.trj'
      include 'wrkspc.trj'

c --- /CONTROL/ variables
      data nmetdat/1/, mfilex/0/, mcoord/1/
      data lcfiles/.FALSE./
      data fileloc/'TRAJLOC.DAT'/
      data filelst/'TRAJ2D.LST'/, filetrj/'TRAJ2D.TRJ'/
      data tegrd/'QATERR.GRD'/, lugrd/'QALUSE.GRD'/
      data gridbna/'QAMETG.BNA'/, luclr/'LUSE.CLR'/
      data pmapt/'UTM     '/,datumt/'WGS-84  '/,utmhemt/'N   '/
      data feastt/0.0/,fnortht/0.0/
      data clat0t/'0.0N            '/,clon0t/'0.0E            '/
      data clat1t/'0.0N            '/,clat2t/'0.0N            '/
      data xmxlen/1.0/,mxnew/1/,xsamlen/1.0/,mxsam/99/
      data nhours/96/

c --- WRKSPC common block
      data nw1/mxnxy/,nw2/mxnxy/

      end


c----------------------------------------------------------------------
c --- Include subroutine packages from CALPUFF
      include 'calpuff.subs'
      include 'calutils.subs'

c --- Include coordinate transformation package
      include 'coordlib.for'
c----------------------------------------------------------------------


c----------------------------------------------------------------------
      subroutine setup
c----------------------------------------------------------------------
c
c --- TRAJ2D    Version: 1.25     Level: 081105                  SETUP
c ---           D. Strimaitis
c
c --- PURPOSE:  Coordinate program set-up operations
c
c --- UPDATES:
c --- V1.1 (061222) to V1.2 (081105)  DGS
c        - Do not echo input control file to list file when control
c          information is provided in the TRAJLOC file (MFILEX=1)
c --- V1.0 (061023) to V1.1 (061222)  DGS
c        - MFILEX option:add RDHDLOC and move TIMESET to COMP
c
c --- INPUTS:
c
c --- OUTPUT:
c
c --- SETUP called by: (main)
c
c --- SETUP calls:      DATETM, COMLINE, RDCF, TFER,
c                       MET1, RDHDLOC
c----------------------------------------------------------------------
      include 'params.trj'
      include 'ctldat.trj'
      include 'qa.trj'

c --- Local string variables
      character*30 dot30
      character*132 filectl
      character*180 command

      data filectl/'TRAJ2D.INP'/
      data dot30/'..............................'/

c --- Report progress
      write(*,*)
      write(*,*)'TRAJ2D Module Running'
      write(*,*)'SETUP PHASE'

c --- Get date and time from system
      call DATETM(rdate,rtime,rcpu)

c --- Overwrite default control file name if provided on command line
      call COMLINE(filectl)

c --- Process instructions from the control file
      open(ictl,file=filectl,status='old')
      call RDCF

c --- Open list file
      open(ilst,file=filelst)

c --- Report initial information to the list file
c --- Write banner
      write(ilst,5) ver,level
5     format(///,26x,'TRAJ2D OUTPUT SUMMARY',/,19x,'VERSION:  ',A12,
     1       ' LEVEL:  ',A12///)

      if(MFILEX.NE.1) then
c ---    Transfer TRAJ2D control file to list file
         write(ilst,'(a80/)')dot30//' Control File Image '//dot30
         call TFER(ictl,ilst)
         write(ilst,'(a80///)')dot30//' Control File Image '//dot30
      endif

c --- Open and read header information from external trajectory
c --- information file (when used)
      if(MFILEX.EQ.1) call RDHDLOC

c --- Open and read specifications of met files
      call MET1

      return
      end

c----------------------------------------------------------------------
      subroutine rdcf
c----------------------------------------------------------------------
c
c --- TRAJ2D    Version: 1.25     Level: 081222                   RDCF
c ---           D. Strimaitis
c
c --- PURPOSE:  Reads control file for TRAJ2D application
c
c --- UPDATES:
c --- V1.1 (061222) to V1.22 (081222)   DGS
c       (1) Add MCOORD
c --- V1.0 (061023) to V1.1 (061222)   DGS
c       (1) Add MFILEX, FILELOC, and NAMET
c       (2) Add limit on number of hours to trace a trajectory (NHOURS)
c       (3) Add MADDCONC
c
c --- INPUTS:
c
c
c --- OUTPUT:
c
c     Common block /CONTROL/
c            x0,y0,namet,nhours,nmetdat,imetdat,lcfiles,
c            mfilex,maddconc,mcoord,
c            ibyrt,ibmot,ibdyt,ibjdt,ibhrt,ibsect,ibdathrt,
c            ieyrt,iemot,iedyt,iejdt,iehrt,iesect,iedathrt,
c            rnlat0t,relon0t,rnlat1t,rnlat2t,
c            iutmznt,feastt,fnortht,pmapt,datumt,datent,utmhemt,
c            clat0t,clon0t,clat1t,clat2t,axbtzt,
c            filemet,filelst,filetrj,fileloc
c
c --- RDCF called by: (main)
c --- RDCF calls:     CASE320, RGET320, NOBLNK320, JULDAY, XTRACTLL
c----------------------------------------------------------------------
      include 'params.trj'
      include 'ctldat.trj'

c --- Local variables
      logical lerror
      character*1 xp,eq
      character*5 case
      character*16 dataset,dataver
      character*320 char320,blnk320,temp320

      data xp/'!'/
      data eq/'='/
      data lerror/.FALSE./

      do i=1,320
         blnk320(i:i)=' '
      enddo

c --- Condition default filenames
      filelst=ADJUSTL(filelst)
      filetrj=ADJUSTL(filetrj)
      fileloc=ADJUSTL(fileloc)

c --- Process first line of control file
      read(ictl,'(2a16)') dataset,dataver
      if(dataset.NE.'TRAJ2D.INP      ') then
         write(*,*)'Error in control file for TRAJ2D application'
         write(*,*)'Expected DATASET name: TRAJ2D.INP'
         write(*,*)'Found    DATASET name: ',dataset
         stop 'HALTED in RDCF'
      endif
      if(dataver.NE.'1.0             ') then
         write(*,*)'Error in control file for TRAJ2D application'
         write(*,*)'Expected Version: 1.2'
         write(*,*)'Found    VERSION: ',dataver
         stop 'HALTED in RDCF'
      endif

c --- Set initial values
c --- Local counters
      nmet=0

c --- Read each record and extract information between !delimiters!
20    char320=blnk320
      read(ictl,'(a320)',end=21) char320

c --- Create upper-case version of line
      klast=LEN_TRIM(char320)
      call CASE320('UPPER',char320,klast)

c --- Position of first 2 delimiters
      ixp1=INDEX(char320,xp)
      if(ixp1.EQ.0) then
c ---    No delimiters, skip this line
         goto 20
      else
         i1=ixp1+1
         i2=klast
         ixp2=INDEX(char320(i1:i2),xp)
         if(ixp2.GT.0) ixp2=ixp1+ixp2
      endif

c --- Must have 2 delimiters
      if(ixp1.GT.ixp2) then
c ---    Problem
         write(*,*)'Error in control file for TRAJ2D application'
         write(*,*)'Single delimiter found on a line:'
         write(*,*)char320(1:klast)
         stop 'HALTED in RDCF'
      endif

c --- Relative position of equals sign within delimiters
      ieq=INDEX(char320(ixp1:ixp2),eq)

c --- Only line with 2 delimiters and no equal sign should be !END!
      if(ieq.EQ.0) then
         if(INDEX(char320,'END').NE.0) then
c ---       Skip this line
            goto 20
         else
c ---       Problem
            write(*,*)'Error in control file for TRAJ2D application'
            write(*,*)'Delimiter-pair found without an assignment:'
            write(*,*)char320(1:klast)
            stop 'HALTED in RDCF'
         endif
      endif

c --- Set range of assigned value
      ieq=ixp1+ieq-1
      i1=ieq+1
      i2=ixp2-1

c --- Go through list of variables
c --------------------------------

c --- Number of input met files
      if(INDEX(char320,'NMETDAT').NE.0) then
         read(char320(i1:i2),'(i)') nmetdat

c --- External trajectory location file name
      elseif(INDEX(char320,'FILELOC').NE.0) then
         read(char320(i1:i2),'(a)') fileloc
         fileloc=ADJUSTL(fileloc)

c --- List file name
      elseif(INDEX(char320,'FILELST').NE.0) then
         read(char320(i1:i2),'(a)') filelst
         filelst=ADJUSTL(filelst)

c --- Output trajectory file name
      elseif(INDEX(char320,'FILETRJ').NE.0) then
         read(char320(i1:i2),'(a)') filetrj
         filetrj=ADJUSTL(filetrj)

c --- Met file name
      elseif(INDEX(char320,'FILEMET').NE.0) then
         nmet=nmet+1
         if(nmet.GT.mxfil) then
            write(*,*)'Error in control file for TRAJ2D application'
            write(*,*)'Too many met files found: ',nmet
            write(*,*)'Increase MXFIL and recompile'
            stop 'HALTED in RDCF'
         endif
         read(char320(i1:i2),'(a)') filemet(nmet)
         filemet(nmet)=ADJUSTL(filemet(nmet))

c --- Case for filename
      elseif(INDEX(char320,'LCFILES').NE.0) then
         temp320=blnk320
         read(char320(i1:i2),'(a)') temp320
         temp320=ADJUSTL(temp320)
         if(temp320(1:1).EQ.'T') then
            lcfiles=.TRUE.
         elseif(temp320(1:1).EQ.'F') then
            lcfiles=.FALSE.
         else
            write(*,*)'Error in control file for TRAJ2D application'
            write(*,*)'Expected LCFILES = T or F:'
            write(*,*)char320(1:klast)
            stop 'HALTED in RDCF'
         endif

c --- Option to add concentration to output filename
      elseif(INDEX(char320,'MADDCONC').NE.0) then
         read(char320(i1:i2),'(i)') maddconc

c --- Type of trajectory coordinates output
      elseif(INDEX(char320,'MCOORD').NE.0) then
         read(char320(i1:i2),'(i)') mcoord

c --- Limit on number of hours in trajectory
      elseif(INDEX(char320,'NHOURS').NE.0) then
         read(char320(i1:i2),'(i)') nhours

c --- External trajectory location file option
      elseif(INDEX(char320,'MFILEX').NE.0) then
         read(char320(i1:i2),'(i)') mfilex

c --- Starting date-time
      elseif(INDEX(char320,'IBYR').NE.0) then
         read(char320(i1:i2),'(i)') ibyrt
      elseif(INDEX(char320,'IBMO').NE.0) then
         read(char320(i1:i2),'(i)') ibmot
      elseif(INDEX(char320,'IBDY').NE.0) then
         read(char320(i1:i2),'(i)') ibdyt
      elseif(INDEX(char320,'IBHR').NE.0) then
         read(char320(i1:i2),'(i)') ibhrt
      elseif(INDEX(char320,'IBSEC').NE.0) then
         read(char320(i1:i2),'(i)') ibsect

c --- Ending date-time
      elseif(INDEX(char320,'IEYR').NE.0) then
         read(char320(i1:i2),'(i)') ieyrt
      elseif(INDEX(char320,'IEMO').NE.0) then
         read(char320(i1:i2),'(i)') iemot
      elseif(INDEX(char320,'IEDY').NE.0) then
         read(char320(i1:i2),'(i)') iedyt
      elseif(INDEX(char320,'IEHR').NE.0) then
         read(char320(i1:i2),'(i)') iehrt
      elseif(INDEX(char320,'IESEC').NE.0) then
         read(char320(i1:i2),'(i)') iesect

c --- Time Zone
      elseif(INDEX(char320,'ABTZ').NE.0) then
         temp320=blnk320
         read(char320(i1:i2),'(a)') temp320
         call NOBLNK320(temp320)
         axbtzt=temp320(1:8)

c --- Location
      elseif(INDEX(char320,'X0').NE.0) then
         call RGET320(char320,i1,i2,x0)
      elseif(INDEX(char320,'Y0').NE.0) then
         call RGET320(char320,i1,i2,y0)
      elseif(INDEX(char320,'NAMET').NE.0) then
         read(char320(i1:i2),'(a)') namet
         namet=ADJUSTL(namet)

c --- Documentation string
      elseif(INDEX(char320,'DOCT').NE.0) then
         read(char320(i1:i2),'(a)') doct
         doct=ADJUSTL(doct)

c --- Coordinate definition
      elseif(INDEX(char320,'PMAP').NE.0) then
         temp320=blnk320
         read(char320(i1:i2),'(a)') temp320
         call NOBLNK320(temp320)
         pmapt=temp320(1:8)
      elseif(INDEX(char320,'FEAST').NE.0) then
         call RGET320(char320,i1,i2,feastt)
      elseif(INDEX(char320,'FNORTH').NE.0) then
         call RGET320(char320,i1,i2,fnortht)
      elseif(INDEX(char320,'IUTMZN').NE.0) then
         read(char320(i1:i2),'(i)') iutmznt
      elseif(INDEX(char320,'UTMHEM').NE.0) then
         temp320=blnk320
         read(char320(i1:i2),'(a)') temp320
         call NOBLNK320(temp320)
         utmhemt=temp320(1:4)
      elseif(INDEX(char320,'RLAT0').NE.0) then
         temp320=blnk320
         read(char320(i1:i2),'(a)') temp320
         call NOBLNK320(temp320)
         clat0t=temp320(1:16)
      elseif(INDEX(char320,'RLON0').NE.0) then
         temp320=blnk320
         read(char320(i1:i2),'(a)') temp320
         call NOBLNK320(temp320)
         clon0t=temp320(1:16)
      elseif(INDEX(char320,'XLAT1').NE.0) then
         temp320=blnk320
         read(char320(i1:i2),'(a)') temp320
         call NOBLNK320(temp320)
         clat1t=temp320(1:16)
      elseif(INDEX(char320,'XLAT2').NE.0) then
         temp320=blnk320
         read(char320(i1:i2),'(a)') temp320
         call NOBLNK320(temp320)
         clat2t=temp320(1:16)
      elseif(INDEX(char320,'DATUM').NE.0) then
         temp320=blnk320
         read(char320(i1:i2),'(a)') temp320
         call NOBLNK320(temp320)
         datumt=temp320(1:8)

c --- Found unexpected variable assignment
      else
         write(*,*)'Error in control file for TRAJ2D application'
         write(*,*)'Unexpected variable assignment found:'
         write(*,*)char320(1:klast)
         stop 'HALTED in RDCF'
      endif

      goto 20

21    continue

c --- Check MFILEX (0 or 1)
      if(mfilex.LT.0 .OR. mfilex.GT.1) then
         write(*,*)'Error in control file for TRAJ2D application'
         write(*,*)'External file option MFILEX: ',mfilex
         write(*,*)'Expected MFILEX = 0 or 1'
         stop 'HALTED in RDCF'
      endif

c --- Check number of files
      if(nmet.NE.nmetdat) then
         write(*,*)'Error in control file for TRAJ2D application'
         write(*,*)'Number of input files found    : ',nmet
         write(*,*)'does not match number expected : ',nmetdat
         stop 'HALTED in RDCF'
      endif

c --- Test information
      if(nmetdat.LE.0) lerror=.TRUE.
      if(LERROR) then
         write(*,*)'Error in control file for TRAJ2D application'
         write(*,*)'Some required information is MISSING or INVALID'
         write(*,*)'nmetdat > 0?         ',nmetdat
         stop 'HALTED in RDCF'
      endif

c --- Apply filename case instruction
      if(LCFILES) then
         case='LOWER'
      else
         case='UPPER'
      endif

      temp320=blnk320
      klast=LEN_TRIM(fileloc)
      temp320(1:klast)=fileloc(1:klast)
      call CASE320(case,temp320,klast)
      fileloc(1:klast)=temp320(1:klast)

      temp320=blnk320
      klast=LEN_TRIM(filelst)
      temp320(1:klast)=filelst(1:klast)
      call CASE320(case,temp320,klast)
      filelst(1:klast)=temp320(1:klast)

      temp320=blnk320
      klast=LEN_TRIM(filetrj)
      temp320(1:klast)=filetrj(1:klast)
      call CASE320(case,temp320,klast)
      filetrj(1:klast)=temp320(1:klast)

      do k=1,nmetdat
         temp320=blnk320
         klast=LEN_TRIM(filemet(k))
         temp320(1:klast)=filemet(k)(1:klast)
         call CASE320(case,temp320,klast)
         filemet(k)(1:klast)=temp320(1:klast)
      enddo

c --- Process remaining control file information (conditional)
      if(mfilex.EQ.0) then
c ---    Construct remaining date variables
         call JULDAY(ilst,ibyrt,ibmot,ibdyt,ibjdt)
         ibdathrt=ibyrt*100000+ibjdt*100+ibhrt
         call JULDAY(ilst,ieyrt,iemot,iedyt,iejdt)
         iedathrt=ieyrt*100000+iejdt*100+iehrt
c --- Process lat/lon character variables to extract content
         call XTRACTLL(ilst,'LAT ',clat1t,rnlat1t)
         call XTRACTLL(ilst,'LAT ',clat2t,rnlat2t)
         call XTRACTLL(ilst,'LAT ',clat0t,rnlat0t)
         call XTRACTLL(ilst,'LON ',clon0t,relon0t)
      endif

      return
      end

c----------------------------------------------------------------------
      subroutine met1
c----------------------------------------------------------------------
c
c --- TRAJ2D    Version: 1.25     Level: 090224                   MET1
c ---           D. Strimaitis
c
c --- PURPOSE:  Read header records of the DA wind data files
c
c --- UPDATES:
c --- V1.22 (081222) to V1.24 (090224)   DGS
c       (1) Avoid blank lat/lon strings in XTRACTLL call
c --- V1.2 (081105) to V1.22 (081222)   DGS
c       (1) Compute time zone difference from met data to trajectory
c --- V1.0 (061023) to V1.2 (081105)   DGS
c       (1) Add the 1-D WIND1D file features, identified by MODE
c           (allows 1D met data from measurements or simple models)
c
c --- INPUTS:
c     Common/CONTROL/
c      nmetdat,filemet
c     Parameters:
c           MXFIL, MXNZP1, MXSS, MXUS, MXPS, ILST, IINP
c
c --- OUTPUT:
c     Common/WINDHD/mode,
c      krecl,nheader(mxfil),iconfig(mxfil),ncoml(mxfil),
c      nx,ny,nz,dgrid,dgridi,xorig,yorig,
c      nsstn,nustn,kwat1,kwat2,zface(mxnzp1),
c      xsstn(mxss),ysstn(mxss),xustn(mxus),yustn(mxus),
c      z0a(mxnx,mxny),ilandua(mxnx,mxny),eleva(mxnx,mxny),
c      xlaia(mxnx,mxny),irlg(mxfil),nsecdt,hrdt,ihrmet2tr,
c      ibyr(mxfil),ibmo(mxfil),ibdy(mxfil),ibjd(mxfil),ibhr(mxfil),
c      ibsec(mxfil),ibdathr(mxfil),
c      ieyr(mxfil),iemo(mxfil),iedy(mxfil),iejd(mxfil),iehr(mxfil),
c      iesec(mxfil),iedathr(mxfil),
c      rnlat0,relon0,rnlat1,rnlat2,
c      iutmzn,feast,fnorth,pmap,datum,daten,utmhem,
c      clat0,clon0,clat1,clat2,axbtz,
c      dataver,dataset,datamod
c
c      Output Files
c         QATERR.GRD - gridded terrain elevations (mMSL)
c         QALUSE.GRD - gridded land use
c         QAMETG.BNA - borders for the METEOROLOGICAL grid domain
c         LUSE.CLR   - color scale for land use
c
c --- MET1 called by:  SETUP
c --- MET1 calls:      JULDAY, XTRACTLL, RDR2D, RDI2D, GRDOUTR, GRDOUTI
c----------------------------------------------------------------------
      include 'params.trj'
      include 'ctldat.trj'
      include 'windhd.trj'
      include 'wrkspc.trj'

c --- Local cariables
      character*40 fmt
      character*8 clabel
      real elev(mxnx,mxny)
      integer ilandu(mxnx,mxny)

      ierr=0

c --- Set GRD format for IMAGE files to SURFER 8
      img=0

c --- Loop over met files
      do k=1,nmetdat
         io=iinp+k-1
         klen=LEN_TRIM(filemet(k))

c ---    Obtain record length
         open(io,file=filemet(k),access='TRANSPARENT')
         read(io) krecl
         close(io)

c ---    Open file as Direct Access
         open(io,file=filemet(k),access='DIRECT',
     &        form='UNFORMATTED',recl=krecl)

c ---    Get enough information from the first record to tell
c ---    if this is a 1D or 2D file, requiring different reads
         read(io,rec=1) irecl,dataset
         if(dataset.EQ.'WIND1D.DAT      ') then
            mode=1
         elseif(dataset.EQ.'WIND2D.DAT      ') then
            mode=2
         else
            write(*,*)'MET1: Met dataset name invalid: ',dataset
            write(*,*)'   Expected: WIND1D.DAT or WIND2D.DAT'
            write(*,*)'      Found: ',clabel
            stop
         endif


         if(mode.EQ.1) then
c ---       1D Record Structure
c ---       -------------------
c ---       First record
            read(io,rec=1) krecl,dataset,ncoml(k),
     &                     nheader(k),iconfig(k)
c ---       Header Documentation records
c ---       Doc Rec 1
            read(io,rec=2) dataver
c ---       Doc Rec 2
            read(io,rec=3) datamod(1:32)
c ---       Doc Rec 3
            read(io,rec=4) datamod(33:64)
c ---       Model configuration records
            irec=iconfig(k)
            read(io,rec=irec) nsecdt,irlg(k),ibyr(k),ibmo(k),ibdy(k),
     &                        ibhr(k),ibsec(k)
            irec=irec+1
            read(io,rec=irec) ieyr(k),iemo(k),iedy(k),
     &                        iehr(k),iesec(k),axbtz
            irec=irec+1
            read(io,rec=irec) nx,ny,dgrid,xorig,yorig,feast,fnorth
            irec=irec+1
            read(io,rec=irec) pmap,datum,daten
            irec=irec+1
            read(io,rec=irec) utmhem,iutmzn
            irec=irec+1
            read(io,rec=irec) clat0,clon0
            irec=irec+1
            read(io,rec=irec) clat1,clat2
            nz=1
c ---       One data record per time
            nrecord=nheader(k)+irlg(k)
         else
c ---       2D Record Structure
c ---       -------------------
c ---       First record
            read(io,rec=1) irecl,dataset,dataver,datamod,
     &                     ncoml(k),nheader(k),iconfig(k),
     &                     ncalmet,method,layerlo,layerup,
     &                     heightlo,heightup,mavg
c ---       Model configuration record
            read(io,rec=iconfig(k)) 
     1         ibyr(k),ibmo(k),ibdy(k),ibhr(k),ibsec(k),
     2         ieyr(k),iemo(k),iedy(k),iehr(k),iesec(k),
     3         axbtz,irlg(k),nsecdt,
     4         nx,ny,nz,dgrid,xorig,yorig,
     5         nsstn,nustn,kwat1,kwat2,
     6         pmap,datum,daten,feast,fnorth,utmhem,iutmzn,
     7         clat0,clon0,clat1,clat2
c ---       Two data records per time
            nrecord=nheader(k)+2*irlg(k)
         endif

         call JULDAY(ilst,ibyr(k),ibmo(k),ibdy(k),ibjd(k))
         call JULDAY(ilst,ieyr(k),iemo(k),iedy(k),iejd(k))
         ibdathr(k)=ibyr(k)*100000+ibjd(k)*100+ibhr(k)
         iedathr(k)=ieyr(k)*100000+iejd(k)*100+iehr(k)

c ---    Reciprocal grid cell size
         dgridi=1./dgrid

cc ---    Number of comment lines (132 characters) per record
c         num=irecl/132
cc ---    Total number of output records for comments
c         ntot=ncoml/num
c         if((ntot*num).LT.ncoml) ntot=ntot+1
cc ---    Number of comment lines in last record
c         nlast=ncoml-(ntot-1)*num

c ---    Report selected information to list file
         write(ilst,*)
         write(ilst,*) 'Selected output header information'
         write(ilst,*) '----------------------------------'
         write(ilst,*) 'Met Data File: ',k,filemet(k)(1:klen)
         write(ilst,*)
         write(ilst,*) 'Record structure'
         write(ilst,*) 'number of bytes/record       = ',krecl
         write(ilst,*) 'number of header records     = ',nheader(k)
         write(ilst,*) 'number of c132 comment lines = ',ncoml(k)
         write(ilst,*) 'configuration info record    = ',iconfig(k)
         write(ilst,*) 'total number of records      = ',nrecord
         write(ilst,*)
         write(ilst,*) 'Met Model Grid'
         write(ilst,*) 'nx,ny,nz,dx = ',nx,ny,nz,dgrid
         write(ilst,*) 'xorig,yorig = ',xorig,yorig
         write(ilst,*) 'pmap,datum  = ',pmap,datum
         write(ilst,*) 'clat0,clon0 = ',clat0,clon0
         write(ilst,*) 'clat1,clat2 = ',clat1,clat2
         write(ilst,*)
         write(ilst,*) 'Data start at'
         write(ilst,*) 'YYYY MM DD HH SSSS = ',ibyr(k),ibmo(k),ibdy(k),
     &                  ibhr(k),ibsec(k)
         write(ilst,*)
         write(ilst,*) 'Data end at'
         write(ilst,*) 'YYYY MM DD HH SSSS = ',ieyr(k),iemo(k),iedy(k),
     &                  iehr(k),iesec(k)
         write(ilst,*)
         write(ilst,*) 'Time Zone = ',axbtz
         write(ilst,*) 'Time-step = ',nsecdt
         write(ilst,*) 'Number of periods = ',irlg(k)
         write(ilst,*)

c ---    Read the timestamp from the last data record in the file
         read(io,rec=nrecord) clabel,ndathrb,nsecb,ndathre,nsece
         write(ilst,*)
         write(ilst,*)'Last data record is for: ',clabel
         write(ilst,*)'                   from: ',ndathrb,nsecb
         write(ilst,*)'                     to: ',ndathre,nsece

c ---    Test data that should be the same
         call QAMET(k,kerr)
         ierr=MAX(ierr,kerr)
      enddo

      if(ierr.GT.0) then
         write(*,*)'MET1: Problems found in met files'
         stop 'HALTED -- see list file'
      endif

c --- Set time shift due to any difference in time zone
c --- Trajectory time zone
      call UTCBASR(axbtzt,xbtzt)
      ibtzt=NINT(xbtzt)
c --- Met time zone
      call UTCBASR(axbtz,xbtz)
      ibtz=NINT(xbtz)
c --- Hours to ADD to MET time to align with Traj time
      ihrmet2tr=ibtz-ibtzt

c --- Process lat/lon character variables to extract content
      if(clat1.EQ.'                ') clat1='0.0N'
      if(clat2.EQ.'                ') clat2='0.0N'
      if(clat0.EQ.'                ') clat0='0.0N'
      if(clon0.EQ.'                ') clon0='0.0E'
      call XTRACTLL(ilst,'LAT ',clat1,rnlat1)
      call XTRACTLL(ilst,'LAT ',clat2,rnlat2)
      call XTRACTLL(ilst,'LAT ',clat0,rnlat0)
      call XTRACTLL(ilst,'LON ',clon0,relon0)

c --- Create plot files of domain fields (k=1)
c --------------------------------------------
c --- Plot entire grid
      nbx=1
      nby=1
      nex=nx
      ney=ny

c --- Set /grid/ information in km
      xorgkm=xorig*.001
      yorgkm=yorig*.001
      delkm=dgrid*.001

c --- Set Upper Right met grid corner (km)
c --- (Lower Left is xorg,yorg)
      xUR=xorgkm+FLOAT(nx)*delkm
      yUR=yorgkm+FLOAT(ny)*delkm

      if(mode.EQ.2) then
c ---    Terrain
         nrec=nheader(1)-1
         call RDR2D(io,nrec,elev,wrk1,mxnx,mxny,nx,ny,clabel,
     &              idum,idum,idum,idum)
c ---    Check that record label matches expected label
         if(clabel.ne.'ELEV    ') then
            write(*,*)'MET1: Did not find ELEV record'
            write(*,*)'      Found: ',clabel
            stop
         endif
c ---    Write grid
         fmt='(10000(1pe11.4,1x))'
         call GRDOUTR(iplt,tegrd,elev,fmt,0,nbx,nby,nex,ney,
     &                delkm,xorgkm,yorgkm)
c ---    Land Use
         nrec=nheader(1)-2
         call RDI2D(io,nrec,ilandu,iwrk1,mxnx,mxny,nx,ny,clabel,
     &              idum,idum,idum,idum)
c ---    Check that record label matches expected label
         if(clabel.ne.'ILANDU  ') then
            write(*,*)'MET1: Did not find ILANDU record'
            write(*,*)'      Found: ',clabel
            stop
         endif
c ---    Write grid
         fmt='(10000(i3,2x))'
         call GRDOUTI(iplt,lugrd,ilandu,fmt,10.,100.,img,
     &                nbx,nby,nex,ney,delkm,xorgkm,yorgkm)
      endif

c --- Set up default land use color file in this directory
      open(iplt,file=luclr,status='REPLACE')
      write(iplt,'(a)') 'ColorMap 1 1'
      write(iplt,'(a)') '    0.000000 255 255   0'
      write(iplt,'(a)') '   11.000000 255 255   0'
      write(iplt,'(a)') '   11.000000 204 255 102'
      write(iplt,'(a)') '   22.100000 204 255 102'
      write(iplt,'(a)') '   22.100000 160 255 160'
      write(iplt,'(a)') '   33.100000 160 255 160'
      write(iplt,'(a)') '   33.100000  80 255  80'
      write(iplt,'(a)') '   44.200000  80 255  80'
      write(iplt,'(a)') '   44.200000 153 255 255'
      write(iplt,'(a)') '   50.000000 153 255 255'
      write(iplt,'(a)') '   50.000000  60 204 255'
      write(iplt,'(a)') '   55.400000  60 204 255'
      write(iplt,'(a)') '   55.400000 204 153 204'
      write(iplt,'(a)') '   66.500000 204 153 204'
      write(iplt,'(a)') '   66.500000 255 204 153'
      write(iplt,'(a)') '   77.600000 255 204 153'
      write(iplt,'(a)') '   77.600000 255 255 204'
      write(iplt,'(a)') '   88.700000 255 255 204'
      write(iplt,'(a)') '   88.700000 255 255 255'
      write(iplt,'(a)') '   99.029557 255 255 255'
      write(iplt,'(a)') '   99.029557 255   0   0'
      write(iplt,'(a)') '  100.000000 255   0   0'
      close(iplt)

c --- Met grid boundary [BNA format]
      open(iplt,file=gridbna,status='REPLACE')
      write(iplt,'(a15)') '" Met","grid",5'
      write(iplt,'(f12.4,a1,f12.4)') xorgkm,',',yorgkm
      write(iplt,'(f12.4,a1,f12.4)') xorgkm,',',yUR
      write(iplt,'(f12.4,a1,f12.4)') xUR, ',',yUR
      write(iplt,'(f12.4,a1,f12.4)') xUR, ',',yorgkm
      write(iplt,'(f12.4,a1,f12.4)') xorgkm,',',yorgkm
      close(iplt)

      return
      end

c----------------------------------------------------------------------
      subroutine qamet(k,ierr)
c----------------------------------------------------------------------
c
c --- TRAJ2D    Version: 1.25     Level: 081105                  QAMET
c ---           D. Strimaitis
c
c --- PURPOSE:  QA selected header records of the 2D wind data
c               files
c
c --- UPDATES:
c --- V1.0 (061023) to V1.2 (081105)   DGS
c       (1) Add MODE to constrain all files to be either 1-D WIND1D
c           or 2-D (not mixed)
c
c --- INPUTS:
c            K - integer  - Index of current met file
c     Parameters:
c           MXFIL, ILST
c
c --- OUTPUT:
c         IERR - integer  - Error indicator
c
c
c --- QAMET called by:  MET1
c --- QAMET calls:      none
c----------------------------------------------------------------------
      include 'params.trj'
      include 'windhd.trj'

      ierr=0
      ierr1=0
      ierr2=0
      ierr3=0

      if(k.EQ.1) then
c ---    First file:  set values for comparisions
         mode1=mode
         axbtz1=axbtz
         nsecdt1=nsecdt
         nx1=nx
         ny1=ny
         nz1=nz
         dgrid1=dgrid
         xorig1=xorig
         yorig1=yorig
         pmap1=pmap
         datum1=datum
         feast1=feast
         fnorth1=fnorth
         utmhem1=utmhem
         iutmzn1=iutmzn
         clat01=clat0
         clon01=clon0
         clat11=clat1
         clat21=clat2
      else
c ---    Compare values for with those from the first file
         if(axbtz1.NE.axbtz) ierr1=1
         if(nsecdt1.NE.nsecdt) ierr1=1
         if(mode1.NE.mode) ierr2=1
         if(nx1.NE.nx) ierr2=1
         if(ny1.NE.ny) ierr2=1
         if(nz1.NE.nz) ierr2=1
         if(dgrid1.NE.dgrid) ierr2=1
         if(xorig1.NE.xorig) ierr2=1
         if(yorig1.NE.yorig) ierr2=1
         if(pmap1.NE.pmap) ierr3=1
         if(datum1.NE.datum) ierr3=1
         if(pmap1.EQ.'UTM     ') then
            if(utmhem1.NE.utmhem) ierr3=1
            if(iutmzn1.NE.iutmzn) ierr3=1
         else
            if(feast1.NE.feast) ierr3=1
            if(fnorth1.NE.fnorth) ierr3=1
            if(clat01.NE.clat0) ierr3=1
            if(clon01.NE.clon0) ierr3=1
            if(clat11.NE.clat1) ierr3=1
            if(clat21.NE.clat2) ierr3=1
         endif
      endif

c --- Report problems
      if(ierr1.EQ.1) then
         ierr=1
         write(ilst,*)
         write(ilst,*) 'QAMET: Inconsistent time in file ',k
         write(ilst,*) 'First file:'
         write(ilst,*) 'Time Zone = ',axbtz1
         write(ilst,*) 'Time-step = ',nsecdt1
         write(ilst,*) 'This file:'
         write(ilst,*) 'Time Zone = ',axbtz
         write(ilst,*) 'Time-step = ',nsecdt
      endif
      if(ierr2.EQ.1) then
         ierr=1
         write(ilst,*)
         write(ilst,*) 'QAMET: Inconsistent met grid in file ',k
         write(ilst,*) 'First file:'
         write(ilst,*) 'mode (1=1D, 2=2D) = ',mode1
         write(ilst,*) 'nx,ny,nz,dx = ',nx1,ny1,nz1,dgrid1
         write(ilst,*) 'xorig,yorig = ',xorig1,yorig1
         write(ilst,*) 'This file:'
         write(ilst,*) 'mode (1=1D, 2=2D) = ',mode
         write(ilst,*) 'nx,ny,nz,dx = ',nx,ny,nz,dgrid
         write(ilst,*) 'xorig,yorig = ',xorig,yorig
      endif
      if(ierr3.EQ.1) then
         ierr=1
         write(ilst,*)
         write(ilst,*) 'QAMET: Inconsistent coordinates in file ',k
         write(ilst,*) 'First file:'
         write(ilst,*) 'pmap,datum  = ',pmap1,datum1
         if(pmap1.EQ.'UTM     ') then
            write(ilst,*) 'iutmzn,utmhem = ',iutmzn1,utmhem1
         else
            write(ilst,*) 'feast,fnorth = ',feast1,fnorth1
            write(ilst,*) 'clat0,clon0  = ',clat01,clon01
            write(ilst,*) 'clat1,clat2  = ',clat11,clat21
         endif
         write(ilst,*) 'This file:'
         write(ilst,*) 'pmap,datum  = ',pmap,datum
         if(pmap1.EQ.'UTM     ') then
            write(ilst,*) 'iutmzn,utmhem = ',iutmzn,utmhem
         else
            write(ilst,*) 'feast,fnorth = ',feast,fnorth
            write(ilst,*) 'clat0,clon0  = ',clat0,clon0
            write(ilst,*) 'clat1,clat2  = ',clat1,clat2
         endif
      endif

      return
      end

c----------------------------------------------------------------------
      subroutine RDMET(idathrm,isecm,umet,vmet,
     &                 kdathre,ksece,kdathrb,ksecb,idone)
c----------------------------------------------------------------------
c
c --- TRAJ2D    Version: 1.25     Level: 090407                  RDMET
c ---           D. Strimaitis
c
c --- PURPOSE:  Read data records from the 2D wind data files
c
c --- UPDATES:
c --- V1.22 (081222) to V1.25 (090407)   DGS
c       (1) Define the EMPTY string from the ASCII code for NULL
c           rather than as the string of nulls (an editor replaced
c           the nulls with blanks in previous version)
c --- V1.21 (081124) to V1.22 (081222)   DGS
c       (1) Adjust met times for any difference in time zones
c       (2) Treat met periods that are shorter than the update period
c           as when data are snapshots
c --- V1.2 (081105) to V1.21 (081124)   DGS
c       (1) Recognize W1D label for interpolated records
c --- V1.0 (061023) to V1.2 (081105)   DGS
c       (1) Add the 1-D WIND1D file features, identified by MODE,
c           and distribute single u,v to all cells
c           (allows 1D met data from measurements or simple models)
c       (2) Modify message about not finding met period
c
c --- INPUTS:
c       IDATHRM - integer       - Mid-pt date/hour of required
c                                 data (YYYYJJJHH)
c         ISECM - integer       - Mid-pt second of required data (SSSS)
c
c     Parameters:
c           MXFIL, ILST
c
c     Common/CONTROL/
c      nmetdat
c     Common/WINDHD/mode,
c      mode,krecl,nheader(mxfil),iconfig(mxfil),ncoml(mxfil),
c      nx,ny,nz,dgrid,xorig,yorig,
c      irlg(mxfil),nsecdt,hrdt,ihrmet2tr
c      ibyr(mxfil),ibmo(mxfil),ibdy(mxfil),ibjd(mxfil),ibhr(mxfil),
c      ibsec(mxfil),ibdathr(mxfil),
c      ieyr(mxfil),iemo(mxfil),iedy(mxfil),iejd(mxfil),iehr(mxfil),
c      iesec(mxfil),iedathr(mxfil)
c     Common/WRKSPC/
c      wrk1(mxnx,mxny),nw1
c
c --- OUTPUT:
c     UMET(mxnx,mxny) - real    - U component of the wind (m/s)
c     VMET(mxnx,mxny) - real    - V-component of the wind (m/s)
c             KDATHRE - integer - End date/hour of data (YYYYJJJHH)
c               KSECE - integer - End second of data (SSSS)
c             KDATHRB - integer - Begin date/hour of data (YYYYJJJHH)
c               KSECB - integer - Begin second of data (SSSS)
c               IDONE - integer - Indicator for period outside of data
c                                 range
c
c        (All times in argment list are in Trajectory Time Zone)
c
c --- RDMET called by:  COMP
c --- RDMET calls:      TPERIOD, DELTSEC, RDR2D, TSHORT
c----------------------------------------------------------------------
      include 'params.trj'
      include 'ctldat.trj'
      include 'windhd.trj'
      include 'wrkspc.trj'

      real umet(mxnx,mxny),vmet(mxnx,mxny)
      real uv1d(2,1)
      character*8 clabel,clabexp,clabexpi,empty

c --- Label on W1D records that are interpolated
      data clabexpi/'UV-1DI'/

c --- Assign empty string from null ASCII character
      do i=1,8
         empty(i:i)=ACHAR(0)
      enddo

c --- Set array dimensions to read the U-V pair in 1D files
      nx1d=2
      ny1d=1

c --- Check that enough workspace has been allocated
      nec=nx*ny
      if(nw1.lt.nec)then
         write(ilst,*)'ERROR in subr. RDMET -- insufficient workspace ',
     1   'allocated to work arrays -- Allocated (NW1) = ',nw1,
     2   '   Required (NEC) = ',nec
         write(*,*)
         stop 'Halted in RDMET -- see list file.'
      endif

c --- Make sure MODE is good before using it
      if(mode.LT.1 .OR. mode.GT.2) then
         write(ilst,*)'RDMET: Invalid MODE: ',mode
         write(ilst,*)'Expected 1(1D) or 2(2D)'
         write(*,*)
         write(*,*)'RDMET: Invalid MODE: ',mode
         write(*,*)'Expected 1(1D) or 2(2D)'
         stop
      endif

c --- Initialize IDONE
      idone=0

c --- Shift target time to time zone of met data (whole hours)
      idathrx=idathrm
      isecx=isecm
      if(ihrmet2tr.NE.0) then
c ---    Back-shift limited to 24 hours in INCR
         nhrinc=MAX(-24,-ihrmet2tr)
         call DEDAT(idathrm,iyrx,ijulx,ihrx)
         call INCR(ilst,iyrx,ijulx,ihrx,nhrinc)
c ---    Pick up any residual shift in second application
         if(nhrinc.NE.-ihrmet2tr) then
            nhrinc=-ihrmet2tr-nhrinc
            call INCR(ilst,iyrx,ijulx,ihrx,nhrinc)
         endif
         idathrx=iyrx*100000+ijulx*100+ihrx
      endif

c --- Loop over met files
      do k=1,nmetdat
         io=iinp+k-1
         call TPERIOD(0,idathrx,isecx,ibdathr(k),ibsec(k),
     &               iedathr(k),iesec(k),ilocate)
         if(ilocate.EQ.0) then
c ---       This file contains the time period needed
c ---       Compute no. periods from beg. of first period to target
            call DELTSEC(ibdathr(k),ibsec(k),idathrx,isecx,ndelsec)
            npd=ndelsec/nsecdt
            if(mode.EQ.1) then
c ---          1D Met Data
               nrec0=npd+nheader(k)
c ---          U and V Wind (constant)
               nrec=nrec0+1
               call RDR2D(io,nrec,uv1d,wrk1,mxnx,mxny,nx1d,ny1d,clabel,
     &              kdathrb,ksecb,kdathre,ksece)
c ---          Check for empty record label
               if(clabel.EQ.empty) goto 998
c ---          Check that record label matches expected label
               clabexp='UV-1D'
               if(clabel.ne.clabexp .AND. clabel.ne.clabexpi) goto 999
c ---          Check to see if required date-time has been read
c ---          Treat time periods shorter than update period
               call TSHORT(ilst,nsecdt,kdathrb,ksecb,kdathre,ksece)
               call TPERIOD(0,idathrx,isecx,kdathrb,ksecb,kdathre,
     &                      ksece,ilocate)
               if(ilocate.NE.0) goto 1999
c ---          Transfer U,V to full array
               do iy=1,ny
               do ix=1,nx
                  umet(ix,iy)=uv1d(1,1)
                  vmet(ix,iy)=uv1d(2,1)
               enddo
               enddo
            elseif(mode.EQ.2) then
c ---          2D Met Data
               nrec0=2*npd+nheader(k)
c ---          U Wind 
               nrec=nrec0+1
               call RDR2D(io,nrec,umet,wrk1,mxnx,mxny,nx,ny,clabel,
     &              kdathrb,ksecb,kdathre,ksece)
c ---          Check that record label matches expected label
               clabexp='U-LEV'
               if(clabel.ne.clabexp) goto 999
c ---          Check to see if required date-time has been read
c ---          Treat time periods shorter than update period
               call TSHORT(ilst,nsecdt,kdathrb,ksecb,kdathre,ksece)
               call TPERIOD(0,idathrx,isecx,kdathrb,ksecb,kdathre,
     &                      ksece,ilocate)
               if(ilocate.NE.0) goto 1999

c ---          V Wind
               nrec=nrec0+2
               call RDR2D(io,nrec,vmet,wrk1,mxnx,mxny,nx,ny,clabel,
     &              kdathrb,ksecb,kdathre,ksece)
c ---          Check that record label matches expected label
               clabexp='V-LEV'
               if(clabel.ne.clabexp) goto 999
c ---          Check to see if required date-time has been read
c ---          Treat time periods shorter than update period
               call TSHORT(ilst,nsecdt,kdathrb,ksecb,kdathre,ksece)
               call TPERIOD(0,idathrx,isecx,kdathrb,ksecb,kdathre,
     &                      ksece,ilocate)
               if(ilocate.NE.0) goto 1999
            endif

c ---       Shift met times to time zone of trajectory (whole hours)
            if(ihrmet2tr.NE.0) then
c ---          Back-shift limited to 24 hours in INCR
               nhrinc=MAX(-24,ihrmet2tr)
               call DEDAT(kdathrb,iyrb,ijulb,ihrb)
               call DEDAT(kdathre,iyre,ijule,ihre)
               call INCR(ilst,iyrb,ijulb,ihrb,nhrinc)
               call INCR(ilst,iyre,ijule,ihre,nhrinc)
c ---          Pick up any residual shift in second application
               if(nhrinc.NE.ihrmet2tr) then
                  nhrinc=ihrmet2tr-nhrinc
                  call INCR(ilst,iyrb,ijulb,ihrb,nhrinc)
                  call INCR(ilst,iyre,ijule,ihre,nhrinc)
               endif
               kdathrb=iyrb*100000+ijulb*100+ihrb
               kdathre=iyre*100000+ijule*100+ihre
            endif

            return
         endif
      enddo

c --- Data for period was NOT found
998   write(ilst,*)' Current trajectory is shorter than requested'//
     &             ' due to lack of wind data.'
      idone=1
      return

c --- Write error messages -- incorrect record label read
999   continue
      write(ilst,1001)clabel,clabexp
1001  format(/1x,'ERROR in subr. RDMET -- incorrect record label ',
     1 'read from MET data file'//1x,'Label read     = ',a8/
     2                               1x,'Label expected = ',a8)
      write(ilst,*)'Searching for period mid-pt ',idathrx,isecx
      write(*,*)
      stop 'Halted in RDMET -- see list file.'

1999  continue
      write(ilst,*)'WARNING: RDMET -- current time not found',
     1' where expected in the MET data files'
      write(ilst,*)' -- (Times are in Met Data time zone)'
      write(ilst,*)' -- Current date/hour,sec = ',idathrx,isecx
      write(ilst,*)' -- Last date/hour,sec read = ',kdathrb,ksecb
      write(ilst,*)'                           to ',kdathre,ksece
      idone=1
      return

      end
c----------------------------------------------------------------------
      subroutine rdr2d(iomet,nrec,x,xbuf,mxnx,mxny,nx,ny,clabel,
     &                 ndathrb,nsecb,ndathre,nsece)
c----------------------------------------------------------------------
c
c --- TRAJ2D    Version: 1.25     Level: 061023                   RDR2D
c ---           J. Scire, D. Strimaitis
c
c --- PURPOSE:  Read NX * NY words of a 2-D real array from direct
c               access file
c
c --- INPUTS:
c            IOMET - integer     - Fortran unit number of input file
c             NREC - integer     - Record number
c      XBUF(nx,ny) - real array  - Buffer to hold input data
c      MXNX,MXNY   - integers    - Dimensions of data array
c            NX,NY - integers    - Actual size of grid to read
c
c --- OUTPUT:
c     X(mxnx,mxny) - real array  - Input data array (padded if nec.)
c           CLABEL - character*8 - Variable name
c          NDATHRB - integer     - Beginning date and time (YYYYJJJHH)
c            NSECB - integer     - Beginning seconds (SSSS)
c          NDATHRE - integer     - Ending date and time (YYYYJJJHH)
c            NSECE - integer     - Ending seconds (SSSS)
c
c --- RDR2D called by:  RDMET
c --- RDR2D calls:      none
c----------------------------------------------------------------------
      real x(mxnx,mxny),xbuf(nx,ny)
      character*8 clabel

      if(nx.eq.mxnx.and.ny.eq.mxny)then
c ---    Entire array is being used -- read full grid
         read(iomet,rec=nrec)clabel,ndathrb,nsecb,ndathre,nsece,x

      else
c ---    Only a portion of grid being used -- read and
c ---    Transfer from buffer
         read(iomet,rec=nrec)clabel,ndathrb,nsecb,ndathre,nsece,xbuf
         do j=1,ny
            do i=1,nx
               x(i,j)=xbuf(i,j)
            enddo
         enddo
      endif

      return
      end

c----------------------------------------------------------------------
      subroutine rdi2d(iomet,nrec,ix,ibuf,mxnx,mxny,nx,ny,clabel,
     &                 ndathrb,nsecb,ndathre,nsece)
c----------------------------------------------------------------------
c
c --- TRAJ2D    Version: 1.25     Level: 061023                   RDI2D
c ---           J. Scire, D. Strimaitis
c
c --- PURPOSE:  Read NX * NY words of a 2-D integer array from direct
c               access file
c
c --- INPUTS:
c            IOMET - integer     - Fortran unit number of input file
c             NREC - integer     - Record number
c      IBUF(nx,ny) - real array  - Buffer to hold input data
c        MXNX,MXNY - integers    - Dimensions of data array
c            NX,NY - integers    - Actual size of grid to read
c
c --- OUTPUT:
c    IX(mxnx,mxny) - int  array  - Input data array (padded if nec.)
c           CLABEL - character*8 - Variable name
c          NDATHRB - integer     - Beginning date and time (YYYYJJJHH)
c            NSECB - integer     - Beginning seconds (SSSS)
c          NDATHRE - integer     - Ending date and time (YYYYJJJHH)
c            NSECE - integer     - Ending seconds (SSSS)
c
c --- RDI2D called by:  RDMET
c --- RDI2D calls:      none
c----------------------------------------------------------------------
      integer ix(mxnx,mxny),ibuf(nx,ny)
      character*8 clabel

      if(nx.eq.mxnx.and.ny.eq.mxny)then
c ---    Entire array is being used -- read full grid
         read(iomet,rec=nrec)clabel,ndathrb,nsecb,ndathre,nsece,ix

      else
c ---    Only a portion of grid being used -- read and
c ---    Transfer from buffer
         read(iomet,rec=nrec)clabel,ndathrb,nsecb,ndathre,nsece,ibuf
         do j=1,ny
            do i=1,nx
               ix(i,j)=ibuf(i,j)
            enddo
         enddo
      endif

      return
      end

c----------------------------------------------------------------------
      subroutine comp
c----------------------------------------------------------------------
c
c --- TRAJ2D    Version: 1.25     Level: 090224                   COMP
c ---           D. Strimaitis
c
c --- PURPOSE:  Main computational component of program
c
c --- UPDATES:
c --- V1.23 (081223) to V1.24 (090224)   IWL
c       (1) Write TRJ filenames to output list file
c
c --- V1.22 (081222) to V1.23 (081223)   DGS
c       (1) Swap order of lat/lon trajectory coordinates to LON,LAT
c
c --- V1.2 (081105) to V1.22 (081222)   DGS
c       (1) Add MCOORD option (lat/lon trajectory coordinates)
c       (2) Add full documentation to header of .TRJ files
c
c --- V1.1 (061222) to V1.2 (081105)   DGS
c       (1) Put actual minutes into time label (may be sub-hourly)
c
c --- V1.0 (061023) to V1.1 (061222)   DGS
c       (1) Add FILELOC (TRAJLOC.DAT) option to create a sequence of
c           trajectory files from a list of starting locations/times
c       (2) Add limit to the number of hours in a trajectory (NHOURS)
c       (3) Add title lines to header of trajectory file
c
c --- INPUTS:
c             none
c
c --- OUTPUT:
c             none
c
c --- COMP called by: (main)
c
c --- COMP calls:      GETLOC, TIMESET, TPERIOD, RDMET, WRHDTRJ,
c                      INITPUF, TIMEUP,
c                      GLOBE1, GLOBE
c----------------------------------------------------------------------
      include 'params.trj'
      include 'ctldat.trj'
      include 'datehr.trj'
      include 'puff.trj'
      include 'windhd.trj'
      include 'wrkspc.trj'

c --- Set initial values for 'last' date-time for met data files
      data ndhrmetb/999900100/,ndhrmete/999900101/
      data nsecmetb/0/,nsecmete/0/

c --- Local Variables
      real xbuf(mxnx,mxny),umet(mxnx,mxny),vmet(mxnx,mxny)
      logical lprt, lsubsam

      character*132 filetrjx
      character*16 caccess
      character*8 clabel
      character*1 cma
      data cma/','/

c --- For coordinate transformations
c --- Set 1 for source location remap
      character*8 cmapi1,cmapo1
      character*12 caction1
      character*4 c4hem1
      real*8 vecti1(9),vecto1(9)
c --- Set 2 for trajectory remap
      character*8 datumgeo
      character*8 cmapi2,cmapo2
      character*12 caction2
      character*4 c4hem2
      real*8 vecti2(9),vecto2(9)

c --- Report progress
      write(*,*)'PROCESSING PHASE'

c --- Set 'Source' location in met grid projection
c --- Scale factor for Tangential TM projection
      tmsone=1.00000
c --- Input of coord transform is from Traj2D control file
c --- or TRAJLOC.DAT file
      iutmi1=iutmznt
      if(utmhemt.EQ.'S   ' .AND. iutmi1.LT.900) iutmi1=-iutmi1
      cmapi1=pmapt
      if(cmapi1.EQ.'TTM     ') cmapi1='TM      '
c --- Output of coord transform is to MET grid
      iutmo1=iutmzn
      if(utmhem.EQ.'S   ' .AND. iutmo1.LT.900) iutmo1=-iutmo1
      cmapo1=pmap
      if(cmapo1.EQ.'TTM     ') cmapo1='TM      '
c --- Set conversion vectors
      call GLOBE1(cmapi1,iutmi1,tmsone,rnlat1t,rnlat2t,
     &               rnlat0t,relon0t,feastt,fnortht,
     &               cmapo1,iutmo1,tmsone,rnlat1,rnlat2,rnlat0,relon0,
     &               feast,fnorth,
     &               caction1,vecti1,vecto1)

c --- Convert release location to met map projection
      call GLOBE(ilst,caction1,datumt,vecti1,datum,vecto1,
     &           x0,y0,xkm,ykm,izone,c4hem)
c --- Compute relative grid coordinates
      xgrd=(1000.*xkm-xorig)*dgridi
      ygrd=(1000.*ykm-yorig)*dgridi
c --- Grid cell index
      ixsrc=1.0+xgrd
      iysrc=1.0+ygrd

c --- Report source coordinate conversion
      write(ilst,*)
      write(ilst,*)'Trajectory source location conversion ---'
      write(ilst,*)'Input File   X0,Y0      = ',x0,y0
      write(ilst,*)'Remapped     X0,Y0 (km) = ',xkm,ykm
      write(ilst,*)'Met Grid     X0,Y0      = ',xgrd,ygrd
      write(ilst,*)'Met Grid   IX0,IY0      = ',ixsrc,iysrc
      write(ilst,*)

c --- Demand that source is on the grid
      if(ixsrc.LT.1 .OR. ixsrc.GT.nx .OR.
     &   iysrc.LT.1 .OR. iysrc.GT.ny) then
         write(ilst,*)'Trajectory origin falls outside of the Met Grid'
         stop 'HALTED:  X0,Y0 is not on met grid -- see list file'
      endif

      if(mcoord.EQ.0) then
c ---    Set conversion from met grid projection to lat,lon
c ---    Scale factor for Tangential TM projection
         tmsone=1.00000
c ---    Input of coord transform is from met file
         iutmi2=iutmzn
         if(utmhem.EQ.'S   ' .AND. iutmi2.LT.900) iutmi2=-iutmi2
         cmapi2=pmap
         if(cmapi2.EQ.'TTM     ') cmapi2='TM      '
c ---    Output of coord transform is to geodetic
         iutmo2=0
         cmapo2='LL      '
c ---    Set conversion vectors
         call GLOBE1(cmapi2,iutmi2,tmsone,rnlat1,rnlat2,
     &               rnlat0,relon0,feast,fnorth,
     &               cmapo2,iutmo2,tmsone,rnlat1,rnlat2,rnlat0,relon0,
     &               feast,fnorth,
     &               caction2,vecti2,vecto2)
         datumgeo='WGS-84  '
      endif

c ----------------------------------------------------
c --- Cycle through trajectory applications
c --- MANY if MFILEX=1, ONE if MFILEX=0 (control file)
c ----------------------------------------------------
1     if(mfilex.EQ.1) then
         filetrjx=filetrj
         call GETLOC(filetrjx,idone)
         nlen=len_trim(filetrjx)
         if(idone.GT.0) return
         write(ilst,'(a)')'  TRJ Filename: '//filetrjx(1:nlen)
         open(itrj,file=filetrjx)
      else
         open(itrj,file=filetrj)
         nlen=len_trim(filetrj)
         write(ilst,'(a)')'  TRJ Filename: '//filetrj(1:nlen)
      endif

c --- Set starting time variables
      call TIMESET

c --- Write header to trajectory file
      call WRHDTRJ

c --- Length of period is NSECDT in seconds
      secdt=FLOAT(nsecdt)

c --- Reciprocal of speed required to travel max allowed distance in 1
c --- timestep in s/m (XMXLEN is maximum step during emission, in grid
c --- units; XSAMLEN is maximum advection step in grid units)
      smxi1=secdt/(xmxlen*dgrid)
      smxi2=secdt/(xsamlen*dgrid)
      xsamleni=1.0/xsamlen

c --- Set boundary of computational grid in MET-GRID units
      xb=0.0
      xe=FLOAT(nx)
      yb=0.0
      ye=FLOAT(ny)

c --- Initialize puff-counter to zero
      npuffs=0

c----------------------------------------------------------
c --- BASIC TIME LOOP (nsecdt per step)
c----------------------------------------------------------
      write(iomesg,*)
      write(iomesg,*)'  --- Processing new trajectory'
      write(iomesg,*)'  --- Advection Step (Middle):'
      write(iomesg,*)'  --- YYYYJJJHH SSSS   # Old  # Emitted'
      write(iomesg,*)

c --- Set number of periods very large (run is halted when all
c --- trajectories are complete (no puffs are on grid)
      iarlg=100000
      do 1000 nn=1,iarlg

c --- Determine data to be read for this step by comparing
c --- the time at the mid-point of the step with the period
c --- for the current Met data
      call TPERIOD(0,ndathrm,nsecm,ndhrmetb,nsecmetb,ndhrmete,nsecmete,
     &             ilocate)
      if(ilocate.NE.0) then
         call RDMET(ndathrm,nsecm,umet,vmet,
     &              ndhrmete,nsecmete,ndhrmetb,nsecmetb,idone)
         if(idone.EQ.1) then
c ---       No more met data for these trajectories
            nactive=0
            goto 2000
         endif
      endif

c --- Initialize puff numbers to report to screen this step
      numold=npuffs

c --- Obtain current wind at source position
c --- (change sign of winds to get backward trajectory transport)
      uadv=-umet(ixsrc,iysrc)
      vadv=-vmet(ixsrc,iysrc)
      ws=SQRT(uadv**2+vadv**2)

c --- Release new puffs for this time step if it falls within simulation
      call TPERIOD(0,ndathre,nsece,ibdathrt,ibsect,iedathrt,iesect,
     &             ilocate)
      if(ilocate.EQ.0) then
         call INITPUF(ndathrb,nsecb,ndathre,nsece,secdt,
     &                xgrd,ygrd,ws,smxi1,newpuf)
         noldp=npuffs
         npuffs=npuffs+newpuf
      else
         noldp=npuffs
         newpuf=0
      endif

c --- Show progress
c --- Tracking message for this timestep
      print 7,ndathrm,nsecm,numold,newpuf
7     format('+',6x,i9,i5,3i9)


c----------------------------------------------------------
c --- LOOP OVER PUFFS
c----------------------------------------------------------
      nactive=0

      do 900 ii=1,npuffs
c
c ---    Skip puffs/slugs off the computational grid
         if(ipufcd(ii).ge.99)go to 900

c ---    Count number that are still active
         nactive=nactive+1

c ---    Use puff position to obtain initial
c ---    met grid cell index - make sure that index is in bounds!
         ixs=1.0+xpb(ii)
         iys=1.0+ypb(ii)
         if(ixs.LT.1) then
            ixs=1
         elseif(ixs.GT.nx) then
            ixs=nx
         endif
         if(iys.LT.1) then
            iys=1
         elseif(iys.GT.ny) then
            iys=ny
         endif

c ---    Assign met parameters from cell (change sign of winds to
c ---    get backward trajectory transport)
         uadv=-umet(ixs,iys)
         vadv=-vmet(ixs,iys)
         ws=SQRT(uadv**2+vadv**2)

c ---    Determine the number & length of the sampling steps
         if(ii.le.noldp)then

c ---       OLD PUFF
            iold=1

c ---       Distance that would be traveled during the time step
c           (in grid units)
            nsam=nint(ws*smxi2+1)
            nsam=min0(nsam,mxsam)
            xsami=1.0/float(nsam)
            nsstr=1
            tsamp=secdt*xsami
         else
c
c ---       NEW PUFF
            iold=0

c ---       Set starting sampling step number for this puff
            ipno=irlsnum(ii)
            nsstr=ipno
            nsam=newpuf
            xsami=1.0/float(nsam)
            tsamp=temit0(ii)

c ---       Write puff data at start to trajectory file
            xkm=.001*(xpb(ii)*dgrid+xorig)
            ykm=.001*(ypb(ii)*dgrid+yorig)
            thours=0.
            if(mcoord.EQ.0) then
c ---          Convert x,y to lat,lon
               call GLOBE(ilst,caction2,datum,vecti2,datumgeo,vecto2,
     &                    xkm,ykm,xelon,ynlat,izone,c4hem)
            endif
            call DEDAT(ndhrp(ii),nyrc,njulc,nhrc)
c ---       With commas
            if(mcoord.EQ.0) then
               write(itrj,'(i10,a1,i5.4,a1,2(f13.7,a1),f9.3,a1,
     &                   i7,a1,i2,a1,i5.4)')
     &                            ndhrp(ii),cma,nsecp(ii),cma,
     &                            xelon,cma,ynlat,cma,thours,cma,
     &                            njulc,cma,nhrc,cma,nsecp(ii)
            elseif(mcoord.EQ.1) then
               write(itrj,'(i10,a1,i5.4,a1,2(f9.3,a1),f13.3,a1,
     &                   i7,a1,i2,a1,i5.4)')
     &                            ndhrp(ii),cma,nsecp(ii),cma,
     &                            xkm,cma,ykm,cma,thours,cma,
     &                            njulc,cma,nhrc,cma,nsecp(ii)
            endif
         endif

c ---    Reciprocal of number of seconds in puff emission period
         temiti=1.0/temit0(ii)
c ---    Reciprocal of number of seconds in a sampling step
         tsampi=1.0/tsamp

c ---    Sampling step may need to be adjusted as puff travels among
c ---    grid cells if speeds change sufficiently.  Assign the initial
c ---    sampling step interval to a reference variable.
         reftsamp=tsamp
         tleft=0.0
c ---    Do not use substeps if the number of sampling steps already
c ---    equals the maximum imposed
         lsubsam=.TRUE.
         if(nsam.EQ.mxsam) lsubsam=.FALSE.

c----------------------------------------------------------
c ---    LOOP OVER SAMPLING STEPS
c----------------------------------------------------------
         do 800 ksam=nsstr,nsam

c ---    Initialize sampling step variables
         tsamp=reftsamp+tleft
c ---    Time left in current sampling step
         tleft=tsamp

c ---    Identify puffs that are at the source at the start of the step
c ---    If puff was generated this time-step (IOLD=0), and if this is
c ---    the first sampling step, then a puff touches the source at the
c ---    start of sampling step, and (ifree=0).
         ifree=1
         if(ksam .EQ. nsstr) ifree=iold

c ---    Re-entry for possible substep
500      continue
         tsampi=1.0/tsamp

c ---    Set last position of PUFF
         xold=xpb(ii)
         yold=ypb(ii)
         ixs=1.0+xold
         iys=1.0+yold
         if(ixs.LT.1) then
            ixs=1
         elseif(ixs.GT.nx) then
            ixs=nx
         endif
         if(iys.LT.1) then
            iys=1
         elseif(iys.GT.ny) then
            iys=ny
         endif

c ---    Determine the advective winds
c ---    (change sign of winds toget backward trajectory transport)
         uadv=-umet(ixs,iys)
         vadv=-vmet(ixs,iys)
         ws=SQRT(uadv**2+vadv**2)

c ---    Apply constraint on step-size
         fstep=1.0
         if(LSUBSAM) then
c ---       Step distance in grid units should not exceed XSAMLEN
            fstep=ws*tsamp*dgridi*xsamleni
            if(fstep.GT.1.0) then
c ---          Substep with shorter sampling time is needed
               if(fstep.GE.2.0 .OR. ksam.LT.nsam) then
c ---             Take a full XSAMLEN this sub-step
                  tsamp=tsamp/fstep
                  tsampi=1.0/tsamp
               else
c ---             This is the last step and 1<fstep<2, so divide it
c ---             into two equal timesteps
                  tsamp=0.5*tsamp
                  tsampi=1.0/tsamp
               endif
            endif
         endif
         tleft=tleft-tsamp

c ---    Compute the DX, DY and new coordinates of the puff (in
c ---    met. grid units).
c ---    First, in meters.
         dxm=uadv*tsamp
         dym=vadv*tsamp
         distm=ws*tsamp
c ---    Now, in met. grid units
         dxg=dxm*dgridi
         dyg=dym*dgridi
         xnew=xold+dxg
         ynew=yold+dyg

c ---    Update total time (s) to end of this step
         tmtotb(ii)=tmtotb(ii)-tsamp
c ---    Update position of puff at end of step
         xpb(ii)=xnew
         ypb(ii)=ynew

c ---    Write puff data at sampling step to trajectory file
         xkm=.001*(xpb(ii)*dgrid+xorig)
         ykm=.001*(ypb(ii)*dgrid+yorig)
         thours=tmtotb(ii)/3600.
         if(mcoord.EQ.0) then
c ---       Convert x,y to lat,lon
            call GLOBE(ilst,caction2,datum,vecti2,datumgeo,vecto2,
     &                 xkm,ykm,xelon,ynlat,izone,c4hem)
         endif
         if(tleft.GT.0. .OR. ksam.LT.nsam) then
c ---       With commas
            if(mcoord.EQ.0) then
               write(itrj,'(i10,a1,i5.4,a1,2(f13.7,a1),f9.3)')
     &                            ndhrp(ii),cma,nsecp(ii),cma,
     &                            xelon,cma,ynlat,cma,thours
            elseif(mcoord.EQ.1) then
               write(itrj,'(i10,a1,i5.4,a1,2(f9.3,a1),f13.3)')
     &                            ndhrp(ii),cma,nsecp(ii),cma,
     &                            xkm,cma,ykm,cma,thours
            endif
         else
            call DEDAT(ndathrb,nyrc,njulc,nhrc)
c ---       With commas
            if(mcoord.EQ.0) then
               write(itrj,'(i10,a1,i5.4,a1,2(f13.7,a1),f9.3,a1,
     &                   i7,a1,i2,a1,i5.4)')
     &                            ndhrp(ii),cma,nsecp(ii),cma,
     &                            xelon,cma,ynlat,cma,thours,cma,
     &                            njulc,cma,nhrc,cma,nsecb
            elseif(mcoord.EQ.1) then
               write(itrj,'(i10,a1,i5.4,a1,2(f9.3,a1),f13.3,a1,
     &                   i7,a1,i2,a1,i5.4)')
     &                            ndhrp(ii),cma,nsecp(ii),cma,
     &                            xkm,cma,ykm,cma,thours,cma,
     &                            njulc,cma,nhrc,cma,nsecb
            endif
         endif

c -----------------------------
c ---    Clean up for next step
c -----------------------------
c ---    Check if new position is off computational grid (reset code)
c ---    Test center of PUFF:
         if(xnew.lt.xb.or.xnew.gt.xe.or.
     1      ynew.lt.yb.or.ynew.gt.ye)then
            ipufcd(ii)=99
            go to 801
         endif
c ---    Check if time limit has been reached (reset code)
         if(thours.LE.-nhours) then
            ipufcd(ii)=99
            go to 801
         endif

         if(LSUBSAM) then
c ---       Check for incomplete sampling step
            if(tleft.GE.1.0 .AND. ksam.EQ.nsam) then
c ---          Last sampling step, so use up remaining sampling time
c ---          by taking another substep
               tsamp=tleft
               goto 500
            elseif(tleft.GT.tsamp) then          
c ---          Time remaining in this step exceeds the time used in
c ---          the current sub-step, so take another substep.
               tsamp=tleft
               goto 500
            endif
c ---       Otherwise, continue on to the next sampling step, and add
c ---       whatever time is left to the reference sampling step time
         endif

800      continue

801      continue

c ---    Close puff loop
900      CONTINUE

c --- Decrement date and time or exit
      if(nactive.GT.0) then
         call TIMEUP(-1)
      else
         goto 2000
      endif

1000  continue

c --- Close current trajectory file
2000  close(itrj)

c --- Next application (conditional)
      if(MFILEX.EQ.1) goto 1

      return
      end

c----------------------------------------------------------------------
      subroutine case320(mode,c320,nlim)
c----------------------------------------------------------------------
c
c --- TRAJ2D    Version: 1.25     Level: 061015                CASE320
c ---           D. Strimaitis
c
c --- PURPOSE:  Convert all letters within a character string to 
c               either upper or lower case
c
c --- INPUTS:
c
c             MODE - character*5       - Output directive for either 
c                                        upper or lower case
c             C320 - character*320     - Input character string
c             NLIM - integer           - Length of string (characters)
c
c --- OUTPUT:
c
c             C320 - character*320     - Output character string with
c                                        uniform case
c
c --- CASE320 called by: (utility)
c --- CASE320 calls:      IACHAR, CHAR
c----------------------------------------------------------------------

      character*320 c320
      character*5 mode

      if(nlim.GT.320) then
         write(*,*)'ERROR in Subr. CASE320 - string is too long'
         write(*,*)'Maximum length  : 320'
         write(*,*)'Requested length: ',nlim
         stop 'Program HALTED'
      endif

      if(mode.EQ.'UPPER' .OR. mode.EQ.'upper') then
         do k=1,nlim
            ia=IACHAR(c320(k:k))
            if(ia.GE.97 .AND. ia.LE.122) then
               ia=ia-32
               c320(k:k)=CHAR(ia)
            endif
         enddo

      elseif(mode.EQ.'LOWER' .OR. mode.EQ.'lower') then
         do k=1,nlim
            ia=IACHAR(c320(k:k))
            if(ia.GE.65 .AND. ia.LE.90) then
               ia=ia+32
               c320(k:k)=CHAR(ia)
            endif
         enddo
      else
         write(*,*)'ERROR in Subr. CASE320 - invalid output mode'
         write(*,*)'Expected UPPER or LOWER'
         write(*,*)'Found: ',mode
         stop 'Program HALTED'
      endif

      return
      end

c----------------------------------------------------------------------
      subroutine noblnk320(c320)
c----------------------------------------------------------------------
c
c --- TRAJ2D    Version: 1.25     Level: 061015              NOBLNK320
c ---           D. Strimaitis
c
c --- PURPOSE:  Remove all blanks from string (left-justify)
c
c --- INPUTS:
c
c             C320 - character*320     - Input character string
c
c --- OUTPUT:
c
c             C320 - character*320     - Output character string with
c
c --- NOBLNK320 called by: (utility)
c --- NOBLNK320 calls:      ADJUSTL, LEN_TRIM
c----------------------------------------------------------------------

      character*320 c320,d320

c --- Blank out work string
      do i=1,320
         d320(i:i)=' '
      enddo

c --- Remove leading blanks
      c320=ADJUSTL(c320)
      last=LEN_TRIM(c320)

c --- Transfer non-blanks into work string
      n=0
      do i=1,last
         if(c320(i:i).NE.' ') then
            n=n+1
            d320(n:n)=c320(i:i)
         endif
      enddo

c --- Pass work string
      c320=d320

      return
      end

c----------------------------------------------------------------------
      subroutine COLUMN320(c320,nlim,cd,ncol,nc,ic)
c----------------------------------------------------------------------
c
c --- TRAJ2D    Version: 1.25     Level: 061015              COLUMN320
c ---           D. Strimaitis
c
c --- PURPOSE:  Identifies location of all column delimiters in a
c               string and checks for expected number
c
c --- INPUTS:
c
c             C320 - character*320     - Input character string
c             NLIM - integer           - Length of string (characters)
c               CD - character*1       - Column delimiter character
c             NCOL - integer           - Number of columns expected
c
c --- OUTPUT:
c
c               NC - integer           - Number of delimiters
c       IC(0:NLIM) - integer           - Location of delimiters
c
c --- COLUMN320 called by: (utility)
c --- COLUMN320 calls:      none
c----------------------------------------------------------------------

      character*1 cd
      character*320 c320
      integer ic(0:ncol)

      if(nlim.GT.320) then
         write(*,*)'ERROR in Subr. COLUMN320 - string is too long'
         write(*,*)'Maximum length  : 320'
         write(*,*)'Requested length: ',nlim
         stop 'Program HALTED'
      endif

      nc=0
      ic(0)=0
      do k=1,nlim
         if(c320(k:k).EQ.cd) then
            if(nc.LT.ncol) then
               nc=nc+1
               ic(nc)=k
            endif
         endif
      enddo
c --- Last delimiter in line is usually missing
      if(nc.LT.ncol) then
         nc=nc+1
         ic(nc)=nlim+1
      endif
c --- Check number of columns
      if(nc.LT.ncol) then
         write(*,*)'ERROR in Subr. COLUMN320 - '
         write(*,*)'Too few columns in the line'
         write(*,*)'Expected number of columns: ',ncol
         write(*,*)'                     Found: ',nc
         write(*,'(a)')c320(1:nlim)
         stop 'Program HALTED'
      endif

      return
      end

c----------------------------------------------------------------------
      subroutine IRANGE(n,ic,nc,i1,i2)
c----------------------------------------------------------------------
c
c --- TRAJ2D    Version: 1.25     Level: 061015                 IRANGE
c ---           D. Strimaitis
c
c --- PURPOSE:  Extracts starting and ending position of field for a
c               particular column in a CSV line
c
c --- INPUTS:
c
c                N - integer           - Column number for field
c         IC(0:NC) - integer           - Location of commas
c               NC - integer           - Number of commas
c
c --- OUTPUT:
c
c               I1 - integer           - Starting position of field
c               I2 - integer           - Ending position of field
c
c --- IRANGE called by: (utility)
c --- IRANGE calls:      none
c----------------------------------------------------------------------

      integer ic(0:nc)

      i1=ic(n-1)+1
      i2=ic(n)-1

      return
      end

c----------------------------------------------------------------------
      subroutine rget320(c320,i1,i2,x)
c----------------------------------------------------------------------
c
c --- TRAJ2D    Version: 1.25     Level: 061015                RGET320
c ---           D. Strimaitis
c
c --- PURPOSE:  Convert character string to a real.  This sub is needed
c               when there is no decimal point.  The LF95 compiler
c               does not do the internal read conversion without the '.'
c               Also, determine when exponential notation is present.
c
c --- INPUTS:
c
c             C320 - character*320     - Input character string
c               i1 - integer           - First character of substring
c               i2 - integer           - Last character of substring
c
c --- OUTPUT:
c
c                X - real              - Output real
c
c --- RGET320 called by: (utility)
c --- RGET320 calls:      none
c----------------------------------------------------------------------
      character*320 c320
      character*20 c20, blnk20
      data blnk20/'                    '/

      if(i1.GT.i2) return

      c20=blnk20
      c20=c320(i1:i2)

c --- Left justify field
      c20=ADJUSTL(c20)

c --- Test for exponential notation
      ie=MAX(INDEX(c20,'e'),INDEX(c20,'E'))

      if(ie.EQ.0) then
c ---    Simple real number
         ipt=INDEX(c20,'.')
         if(ipt.EQ.0) then
c ---       Decimal point NOT found
            ipt=LEN_TRIM(c20)+1
            c20(ipt:ipt)='.'
         endif
c ---    Decimal point in place: internal read OK
         read(c20,'(f)') x

      else
c ---    Exponential notation
         ipt=INDEX(c20,'.')
         if(ipt.EQ.0) then
c ---       Decimal point NOT found
            do k=20,ie+1,-1
               km1=k-1
               c20(k:k)=c20(km1:km1)
            enddo
            c20(ie:ie)='.'
         endif
c ---    Decimal point in place: internal read OK
         read(c20,'(e)') x

      endif

      return
      end



c----------------------------------------------------------------------
      subroutine mlines320(io,char320)
c----------------------------------------------------------------------
c
c --- TRAJ2D    Version: 1.25     Level: 061015              MLINES320
c ---           D. Strimaitis
c
c --- PURPOSE:  Reads additional records from control file format
c               file when an input assignment extends to more than
c               one line
c
c --- INPUTS:
c               IO - integer      - File unit number
c          CHAR320 - char*320     - Current content of variable
c                                   assignment string
c
c --- OUTPUT:
c          CHAR320 - char*320     - Updated content of variable
c                                   assignment string
c
c --- MLINES320 called by: (main)
c --- MLINES320 calls:     NOBLNK320, CASE320
c----------------------------------------------------------------------
      character*1 cm,xp
      character*180 char180,blnk180
      character*320 char320,blnk320,work320

      data cm/','/
      data xp/'!'/

      do i=1,320
         blnk320(i:i)=' '
      enddo
      blnk180=blnk320(1:180)

c --- Remove all characters that precede the first '!'
      work320=blnk320
      work320=char320(INDEX(char320,xp):320)
      char320=ADJUSTL(work320)

10    klast=LEN_TRIM(char320)

c --- Check current assignment string to see if it is complete
      if(char320(klast:klast).EQ.xp) then
c ---    Assignment is complete
         return
      elseif(char320(klast:klast).EQ.cm) then
c ---    Append next line
         char180=blnk180
         read(io,'(a180)',end=999) char180
         work320=blnk320
         work320(1:180)=char180
         call NOBLNK320(work320)
         klast2=LEN_TRIM(work320)
         call CASE320('UPPER',work320,klast2)
         i1=klast+1
         i2=klast+klast2
         if(i2.GT.320) then
            write(*,*)'ERROR -- assignment string exceeds 320 ',
     &                'characters.  Total number of characters ',
     &                'reached: ',i2
            write(*,*)char320
            write(*,*)work320
            stop 'HALTED in subroutine MLINES320'
         endif
         char320(i1:i2)=work320(1:klast2)
         goto 10
      else
         write(*,*)'ERROR -- completed assignment string must end ',
     &             'with ! and a continued assignment string must ',
     &             'end with a comma'
         write(*,*)char320
         stop 'HALTED in subroutine MLINES320'
      endif

999   write(*,*)'ERROR -- End-of-File encountered when completing ',
     &          'assignment string'
      write(*,*)char320
      stop 'HALTED in subroutine MLINES320'

      end


c----------------------------------------------------------------------
      subroutine rhs320(char1,char320)
c----------------------------------------------------------------------
c
c --- TRAJ2D    Version: 1.25     Level: 061015                 RHS320
c ---           D. Strimaitis
c
c --- PURPOSE:  Removes characters from the left-hand side of a
c               string up through "char1", and returns the RHS
c               that remains
c
c --- INPUTS:
c            CHAR1 - char*1       - LHS character limit
c          CHAR320 - char*320     - Current content of variable
c                                   assignment string
c
c --- OUTPUT:
c          CHAR320 - char*320     - Updated content of variable
c                                   assignment string
c
c --- RHS320 called by: (main)
c --- RHS320 calls:     none
c----------------------------------------------------------------------
      character*1 char1
      character*320 char320,work320

      do i=1,320
         work320(i:i)=' '
      enddo

      i1=INDEX(char320,char1)+1
      i2=LEN_TRIM(char320)
      work320=char320(i1:i2)
      char320=ADJUSTL(work320)

      return
      end

c----------------------------------------------------------------------
      subroutine omit320(char1,char320)
c----------------------------------------------------------------------
c
c --- TRAJ2D    Version: 1.25     Level: 061015                OMIT320
c ---           D. Strimaitis
c
c --- PURPOSE:  Removes all instances of a character from a string
c
c --- INPUTS:
c            CHAR1 - char*1       - Character removed
c          CHAR320 - char*320     - Current content of variable
c                                   assignment string
c
c --- OUTPUT:
c          CHAR320 - char*320     - Updated content of variable
c                                   assignment string
c
c --- OMIT320 called by: (main)
c --- OMIT320 calls:     none
c----------------------------------------------------------------------
      character*1 char1
      character*320 char320,work320

      do i=1,320
         work320(i:i)=' '
      enddo

      kk=0
      klast=LEN_TRIM(char320)
      do k=1,klast
         if(char320(k:k).NE.char1) then
            kk=kk+1
            work320(kk:kk)=char320(k:k)
         endif
      enddo
      char320=work320

      return
      end

c----------------------------------------------------------------------
      subroutine tfer(iinp,iout)
c----------------------------------------------------------------------
c
c --- TRAJ2D    Version: 1.25     Level: 061015                   TFER
c ---           D. Strimaitis
c
c --- PURPOSE:  Transfers one file image to another file
c
c --- INPUTS:
c             IINP - integer      - Unit number for input file
c             IOUT - integer      - Unit number for output file
c
c --- OUTPUT:
c
c
c --- TFER called by: (main)
c --- TFER calls:     none
c----------------------------------------------------------------------

c --- Local variables
      character*640 char640,blnk640
      do i=1,640
         blnk640(i:i)=' '
      enddo

c --- Back to start of file
      REWIND(iinp)

c --- Read and write each record
20    char640=blnk640
      read(iinp,'(a640)',end=21) char640
      klast=LEN_TRIM(char640)
      write(iout,'(a)') char640(1:klast)
      goto 20

21    close(iinp)

      return
      end

c----------------------------------------------------------------------
      subroutine fin
c----------------------------------------------------------------------
c
c --- TRAJ2D   Version: 1.25     Level: 061023                     FIN
c ---          D. Strimaitis
c
c --- PURPOSE:  Run termination routine -- compute runtime,
c               write last day processed
c
c --- INPUTS:  none
c
c --- OUTPUT:  none
c
c --- FIN called by:  (main)
c --- FIN calls:      DATETM, GRDAY, JULDAY, DELTT, INDECR, YR4C
c----------------------------------------------------------------------
c --- Include parameters
      include 'params.trj'
      include 'datehr.trj'
      include 'qa.trj'

      character*8 rtime2
      character*10 rdate2

      write(*,*)
      write(*,*)'TERMINATION PHASE'

c --- Last day/hour processed
      write(ilst,5)nyrb,nmob,ndayb,njulb,nhrb,nsecb
5     format(//2x,'LAST MET PERIOD PROCESSED STARTS AT:'/5x,
     1       '  Year: ',i4,2x,'Month: ',i2,3x,'Day: ',i2,3x,
     2       'Julian day: ',i3,3x,'Hour: ',i2,3x,'Second: ',i4)

c --- Get system date & time at end of run
      call DATETM(rdate2,rtime2,rcpu)

c --- Compute runtime
      read(rtime(1:2),10)ihr1
      read(rtime(4:5),10)imin1
      read(rtime(7:8),10)isec1
10    format(i2)
      t1=ihr1*3600.+imin1*60.+isec1

      read(rtime2(1:2),10)ihr2
      read(rtime2(4:5),10)imin2
      read(rtime2(7:8),10)isec2
      t2=ihr2*3600.+imin2*60.+isec2

      if(rdate.eq.rdate2)then
         delt=t2-t1
      else
         read(rdate(1:2),10)imo1
         read(rdate(4:5),10)iday1
         read(rdate(7:10),'(i4)')iyr1
         call julday(ilst,iyr1,imo1,iday1,ijul1)

         read(rdate2(1:2),10)imo2
         read(rdate2(4:5),10)iday2
         read(rdate2(7:10),'(i4)')iyr2
         call julday(ilst,iyr2,imo2,iday2,ijul2)

c ---    Compute no. hours from beg. of first hour of run to
c ---    ending hour of ending day of the run
         call deltt(iyr1,ijul1,ihr1,iyr2,ijul2,ihr2,idelhr)

c ---    Adjust for minutes and seconds
         delt=idelhr*3600.-imin1*60.-isec1+imin2*60.+isec2
      endif

c --- On the PC, the runtime and CPU time are the same
c --- (DATETM provides RCPU = 0.0 on the PC)
      if(rcpu.eq.0.0)rcpu=delt

c --- Report current date
      write(ilst,1402)rtime2,rdate2,delt,rcpu
1402  format(//2x,'End of run -- Clock time: ',a8/
     1         2x,'                    Date: ',a10//
     2         2x,'      Elapsed clock time: ',f10.1,' (seconds)'//
     3         2x,'                CPU time: ',f10.1,' (seconds)')

      return
      end

c----------------------------------------------------------------------
      subroutine timeset
c----------------------------------------------------------------------
c
c --- TRAJ2D   Version: 1.25     Level: 061023                 TIMESET
c ---          D. Strimaitis
c
c --- PURPOSE:  Set date/time variables for the start of the run
c
c --- INPUT:
c
c     Common block /WINDHD/
c        NSECDT
c     Common block /CONTROL/
c        IEYRT, IEMOT, IEDAYT, IEHRT, IESECT
c
c --- OUTPUT:
c
c     Common block /DATEHR/
c        nyrb,nmob,ndayb,njulb,nhrb,nsecb,ndathrb,
c        nyre,nmoe,ndaye,njule,nhre,nsece,ndathre,
c        nyrm,njulm,nhrm,nsecm,ndathrm,
c        nhrind
c        
c
c --- TIMESET called by: SETUP
c --- TIMESET calls:     JULDAY,  GRDAY,  INCRS
c----------------------------------------------------------------------
      include 'params.trj'
c
c --- Include common blocks
      include 'ctldat.trj'
      include 'datehr.trj'
      include 'windhd.trj'

c --- Mark periods with both the beginning and ending time
c --- Length of period is NSEC in seconds
c --- Half of period is NSEC2 seconds
      nsec=-nsecdt
      nsec2=nsec/2

c --- Set ending time of first period
      nyre=ieyrt
      nmoe=iemot
      ndaye=iedyt
      nhre=iehrt
      nsece=iesect
      call JULDAY(io6,nyre,nmoe,ndaye,njule)
      ndathre=nyre*100000+njule*100+nhre

c --- Set time at the middle of first period
      nyrm=nyre
      njulm=njule
      nhrm=nhre
      nsecm=nsece
      call INCRS(io6,nyrm,njulm,nhrm,nsecm,nsec2)
      ndathrm=nyrm*100000+njulm*100+nhrm

c --- Set starting time of first period
      nyrb=nyre
      njulb=njule
      nhrb=nhre
      nsecb=nsece
      call INCRS(io6,nyrb,njulb,nhrb,nsecb,nsec)
      call GRDAY(io6,nyrb,njulb,nmob,ndayb)
      ndathrb=nyrb*100000+njulb*100+nhrb

c --- Use midpoint of period to assign an hour index (1-24)
      nhrind=nhrm+1

      return
      end

c----------------------------------------------------------------------
      subroutine timeup(i)
c----------------------------------------------------------------------
c
c --- TRAJ2D   Version: 1.25     Level: 061023                  TIMEUP
c ---          D. Strimaitis
c
c --- PURPOSE:  Update date/time variables by 1 timestep
c
c --- INPUT:
c               I - integer      - Direction of update
c                                  i=+1  Forward 1 step
c                                  i=-1  Backward 1 step
c
c     Common block /DATEHR/
c        nyrb,nmob,ndayb,njulb,nhrb,nsecb,ndathrb,
c        nyre,nmoe,ndaye,njule,nhre,nsece,ndathre,
c        nyrm,njulm,nhrm,nsecm,ndathrm,
c     Common block /WINDHD/
c        NSECDT
c
c --- OUTPUT:
c
c     Common block /DATEHR/
c        nyrb,nmob,ndayb,njulb,nhrb,nsecb,ndathrb,
c        nyre,nmoe,ndaye,njule,nhre,nsece,ndathre,
c        nyrm,njulm,nhrm,nsecm,ndathrm,
c        nhrind
c        
c
c --- TIMEUP called by: COMP
c --- TIMEUP calls:     JULDAY,  GRDAY,  INCRS
c----------------------------------------------------------------------
      include 'params.trj'
c
c --- Include common blocks
      include 'datehr.trj'
      include 'windhd.trj'

c --- Set the update step (seconds)
      nstep=nsecdt*i

      if(i.EQ.1) then
c ---    Forward 1 step
c ---    Beginning
         nyrb=nyre
         nmob=nmoe
         ndayb=ndaye
         njulb=njule
         nhrb=nhre
         nsecb=nsece
         ndathrb=ndathre
c ---    Mid-point
         call INCRS(io6,nyrm,njulm,nhrm,nsecm,nstep)
         ndathrm=nyrm*100000+njulm*100+nhrm
c ---    Ending
         call INCRS(io6,nyre,njule,nhre,nsece,nstep)
         call GRDAY(io6,nyre,njule,nmoe,ndaye)
         ndathre=nyre*100000+njule*100+nhre

      elseif(i.EQ.-1) then
c ---    Backward 1 step
c ---    Ending
         nyre=nyrb
         nmoe=nmob
         ndaye=ndayb
         njule=njulb
         nhre=nhrb
         nsece=nsecb
         ndathre=ndathrb
c ---    Mid-point
         call INCRS(io6,nyrm,njulm,nhrm,nsecm,nstep)
         ndathrm=nyrm*100000+njulm*100+nhrm
c ---    Beginning
         call INCRS(io6,nyrb,njulb,nhrb,nsecb,nstep)
         call GRDAY(io6,nyrb,njulb,nmob,ndayb)
         ndathrb=nyrb*100000+njulb*100+nhrb

      else
c ---    Argument must be either -1 or 1
         write(*,*)
         write(*,*)'FATAL ERROR in TIMEUP---'
         write(*,*)'The update step is not -1 or 1: ',i
         stop
      endif

c --- Use midpoint of period to assign an hour index (1-24)
      nhrind=nhrm+1

      return
      end


c----------------------------------------------------------------------
      subroutine initpuf(ndhrb,nsecb,ndhre,nsece,dtsec,
     &                   xgrd,ygrd,ws,smxi,newpuf)
c----------------------------------------------------------------------
c
c --- TRAJ2D    Version: 1.25     Level: 061023                INITPUF
c               D. Strimaitis
c
c --- PURPOSE:  Initialize newly released puff variables
c
c --- INPUTS:
c         NDHRB - integer - Julian day, hour beginning current step
c         NSECB - integer - Seconds at beginning of current step
c         NDHRE - integer - Julian day, hour at end of current step
c         NSECE - integer - Seconds at end of current step
c          XGRD - real    - source X-location in met grid units
c          YGRD - real    - source Y-location in met grid units
c            WS - real    - Wind speed at release (m/s)
c          SMXI - real    - Reciprocal of speed required to travel
c                           max allowed distance in 1 timestep (s/m)
c         DTSEC - real    - Length (seconds) of a basic time step
c
c
c --- OUTPUT:
c        NEWPUF - integer - Number of new puffs released
c                           during the current time step
c     Common/PUFF/
c        xpb(mxpuff),ypb(mxpuff),tmtotb(mxpuff),
c        irlsnum(mxpuff),ipufcd(mxpuff),npuffs,
c        ndhrp(mxpuff),nsecp(mxpuff),temit0(mxpuff)
c
c --- INITPUF called by:  COMP
c --- INITPUF calls:      DELTSEC, INCRS
c----------------------------------------------------------------------
      include 'params.trj'
      include 'ctldat.trj'
      include 'puff.trj'

c --- Decode date-time at end
      call DEDAT(ndhre,jyrp,jjulp,jhrp)
      jsecp=nsece

c --- Initialize current number of puffs equal to the number of
c --- old puffs, and set the number of new puffs to zero;
      np=npuffs
      newpuf=0
      npnew=0

c --- Establish emitting time
      ndhr1=ndhrb
      nsec1=nsecb
      ndhr2=ndhre
      nsec2=nsece
c --- Duration of emission period within timestep (seconds)
      call DELTSEC(ndhr1,nsec1,ndhr2,nsec2,ntemit)
      temit=FLOAT(ntemit)

c --- Apply transport constraints
      npnewt=ws*smxi*temit/dtsec+1
      npnewt=amin0(npnewt,mxnew)
      temit=temit/FLOAT(npnewt)
c --- Update puff numbers
      npnew=npnew+npnewt
      newpuf=newpuf+npnewt

c --- Initialize current new puffs
      do j=1,npnewt
         np=np+1
c ---    Fill values
         tmtotb(np)=0.0
         xpb(np)=xgrd
         ypb(np)=ygrd
         temit0(np)=temit
         ipufcd(np)=1
c ---    Set puff identity where IRLSNUM is the puff
c ---    number released from this time step
         irlsnum(np)=npnew-npnewt+j
c ---    Release time
         nsback=-NINT(temit)
         if(j.GT.1) call INCRS(ilst,jyrp,jjulp,jhrp,jsecp,nsback)
         ndhrp(np)=jyrp*100000+jjulp*100+jhrp
         nsecp(np)=jsecp
      enddo

      return
      end

c----------------------------------------------------------------------
      subroutine xtractll(io,type,clatlon,rlatlon)
c----------------------------------------------------------------------
c
c --- TRAJ2D    Version: 1.25     Level: 061023               XTRACTLL
c               D. Strimaitis
c
c --- PURPOSE:  Extract the real latitude or longitude from a character
c               string that contains the N/S or E/W convention
c               character, and express result as either North Latitude
c               or East Longitude
c
c --- INPUTS:
c               IO - integer    - Unit number for list file output
c             TYPE - char*4     - LAT or LON
c          CLATLON - char*16    - Latitude or longitude (degrees), with
c                                 1 character that denotes convention
c                                 (e.g. 'N  45.222' or  '-35.999s')
c
c --- OUTPUT:
c          RLATLON - real       - North Latitude or East Longitude
c                                 (degrees)
c
c --- XTRACTLL called by: (utility)
c --- XTRACTLL calls:     NOBLNK320, CASE320, RGET320
c----------------------------------------------------------------------
      character*320 c320,blnk320
      character*16 clatlon, clatlon2
      character*4 type
      logical ltype

      ltype=.FALSE.

c --- Place string into work string 
      c320(1:16)=clatlon
      do i=17,320
         c320(i:i)=' '
      enddo

c --- Remove blanks enforce upper case
      call NOBLNK320(c320)
      klast=LEN_TRIM(c320)
      call CASE320('UPPER',c320,klast)

c --- Was valid type provided?
      if(type.NE.'LAT ' .AND. type.NE.'LON ') then
         write(io,*) 'XTRACTLL:  FATAL ERROR reported when ',
     &               'extracting Latitude/Longitude'
         write(io,*) 'Invalid type:  ',type
         write(io,*) 'Expected LAT or LON'
         write(*,*)
         stop 'Halted in XTRACTLL -- see list file'
      endif

c --- Interpret valid convention character (N,S,E,W)
      nchar=0
      ichar=0
      ilat=0
      ilon=0
   
      do i=1,klast
         if(c320(i:i).EQ.'N') then
            ilat=1
            ichar=i
            nchar=nchar+1
         elseif(c320(i:i).EQ.'S') then
            ilat=2
            ichar=i
            nchar=nchar+1
         elseif(c320(i:i).EQ.'W') then
            ilon=1
            ichar=i
            nchar=nchar+1
         elseif(c320(i:i).EQ.'E') then
            ilon=2
            ichar=i
            nchar=nchar+1
         endif
      enddo

c --- Was 1 valid character found?
      if(nchar.NE.1) then
         write(io,*) 'XTRACTLL:  FATAL ERROR reported when ',
     &               'extracting Latitude/Longitude'
         write(io,*) 'N,S,E,W character is missing or repeated'
         write(io,*) 'Lat/Lon = ',clatlon
         write(*,*)
         stop 'Halted in XTRACTLL -- see list file'
      endif

c --- Was valid character the right type?
      if(type.EQ.'LAT ' .AND. ilat.EQ.0) ltype=.TRUE.
      if(type.EQ.'LON ' .AND. ilon.EQ.0) ltype=.TRUE.
      if(LTYPE) then
         write(io,*) 'XTRACTLL:  FATAL ERROR reported when ',
     &               'extracting Latitude/Longitude'
         write(io,*) 'N,S,E,W character does not match type'
         write(io,*) 'Lat/Lon = ',clatlon
         write(io,*) 'type    = ',type
         write(*,*)
         stop 'Halted in XTRACTLL -- see list file'
      endif

c --- Remove character from string
      do i=ichar,klast
         ip1=i+1
         c320(i:i)=c320(ip1:ip1)
      enddo

c --- Extract real degrees
      call RGET320(c320,1,klast,rlatlon)

c --- Convert to either N. Lat. or E. Lon., if needed
      if(ilat.EQ.2) then
         rlatlon=-rlatlon
      elseif(ilon.EQ.1) then
         rlatlon=-rlatlon
      endif

c --- Condition longitude to be -180 to +180
      if(ilon.GT.0) then
         if(rlatlon.GT.180.) then
            rlatlon=rlatlon-360.
         elseif(rlatlon.LT.-180.) then
            rlatlon=rlatlon+360.
         endif
      endif

      return
      end

c----------------------------------------------------------------------
      subroutine grdoutr(io,filenam,var,fmt,img,nbx,nby,nex,ney,
     &                   delx,x0km,y0km)
c----------------------------------------------------------------------
c --- TRAJ2D    Version: 1.25     Level: 061023                GRDOUTR
c               D. Strimaitis
c
c --- ADAPTED FROM:
c --- PRTMET    Version: 4.44      Level: 040923               GRDOUTR
c               F. Robe, D. Strimaitis
c
c --- PURPOSE:  Write out a 'GRD' file compatible with SURFER for
c               making plots ---- REAL field version
c
c --- INPUTS:
c             IO - integer                - output unit number
c        FILENAM - character*132          - name of output file
c            VAR - real array (mxnx,mxny) - 2-D variable to plot
c            FMT - character*40           - format for output record
c                                           (set to 'BINARY' if file
c                                            type is binary)
c            IMG - integer                - image file flag
c                                            0: Image uses standard GRD
c                                               or file used for contour
c                                              (data at cell center)
c                                            1: Image uses shifted range
c                                              (data cover entire cell)
c      NBX     - integer                - X Index of the lower left
c                                         gridpoint of subdomain
c      NBY     - integer                - Y Index of the lower left
c                                         gridpoint of subdomain
c      NEX     - integer                - X Index of the lower right
c                                         gridpoint of subdomain
c      NEY     - integer                - Y Index of the upper left
c                                         gridpoint of subdomain
c      DELX    - real                   - Length of grid cell (km)
c      X0KM    - real                   - X coord. of origin (km)
c      Y0KM    - real                   - Y coord. of origin (km)
c
c --- OUTPUT: output file (unit=io)
c
c --- GRDOUTR called by:  MET1
c --- GRDOUTR calls:      none
c
c---------------------------------------------------------------------------
      include 'params.trj'

      real var(mxnx,mxny)
      character*132 filenam
      character*40 fmt
      character*4 cid

      real*8 xlo,xhi,ylo,yhi,zlo,zhi
      integer*2 nx2,ny2
      logical lascii

      data rbig/9.9e+20/
      data cid/'DSBB'/

c --- Check range of cells
      if(nbx.GT.nex .OR. nbx.LT.1 .OR. nex.GT.mxnx .OR.
     &   nby.GT.ney .OR. nby.LT.1 .OR. ney.GT.mxny) then
         write(*,*)
         write(*,*)'GRDOUTR: Invalid grid range requested'
         write(*,*)'nbx,nex = ',nbx,nex
         write(*,*)'nby,ney = ',nby,ney
         write(*,*)'Expected nbx<nex, nby<ney within the range'
         write(*,*)'1 to mxnx = ',mxnx,' and 1 to mxny = ',mxny
         stop
      endif

c --- Check image type option
      if(img.LT.0 .OR. img.GT.1) then
         write(*,*)'GRDOUTR: Invalid IMAGE flag requested = ',img
         write(*,*)'Expected 0 or 1'
         stop
      endif

c --- Set X-Y coordinates of the begin/end gridpoints (km)
      xbgkm=x0km+(nbx-0.5)*delx
      ybgkm=y0km+(nby-0.5)*delx
      xegkm=x0km+(nex-0.5)*delx
      yegkm=y0km+(ney-0.5)*delx

c --- Set logical for either ASCII or BINARY file type
      if(fmt(1:6).EQ.'BINARY') then
         lascii=.FALSE.
      else
         lascii=.TRUE.
      endif

c --- Find minimum and maximum values:
      rmax=-rbig
      rmin=rbig
      do j=nby,ney
         do i=nbx,nex
           if(var(i,j).gt.rmax)rmax=var(i,j)
           if(var(i,j).lt.rmin)rmin=var(i,j)
         enddo
      enddo

c --- Do not produce file if there is no spatial gradient
      if(rmin .EQ. rmax) then
         close(io, status='DELETE')
         return
      endif

c --- Shift (km) from cell center to edge for IMAGE plot
      ds=0.5*delx*FLOAT(img)

      if(LASCII) then
         open(io,file=filenam,status='REPLACE')
c ---    Header
         write(io,'(a4)')'DSAA'
         write(io,'(2i12)')nex-nbx+1,ney-nby+1
         write(io,'(2f12.4)')xbgkm-ds,xegkm+ds
         write(io,'(2f12.4)')ybgkm-ds,yegkm+ds
         write(io,'(2e12.4)')rmin,rmax
c ---    Data records
         do j=nby,ney
            write(io,fmt)(var(i,j),i=nbx,nex)
         enddo

      else

         open(io,file=filenam,form='UNFORMATTED',
     &        access='TRANSPARENT',status='REPLACE')
c ---    Set real*8 components of the header
         xlo=DBLE(xbgkm-ds)
         ylo=DBLE(ybgkm-ds)
         zlo=DBLE(rmin)
         xhi=DBLE(xegkm+ds)
         yhi=DBLE(yegkm+ds)
         zhi=DBLE(rmax)
c ---    Set integer*4 components of the header
         nx2=nex-nbx+1
         ny2=ney-nby+1
c ---    Header
         write(io)cid
         write(io)nx2,ny2
         write(io)xlo,xhi
         write(io)ylo,yhi
         write(io)zlo,zhi
c ---    Data records
         do j=nby,ney
            write(io)(var(i,j),i=nbx,nex)
         enddo
      endif

      close(io)

      return
      end

c----------------------------------------------------------------------
      subroutine grdouti(io,filenam,ivar,fmt,zlo,zhi,img,
     &                   nbx,nby,nex,ney,delx,x0km,y0km)
c----------------------------------------------------------------------
c --- TRAJ2D    Version: 1.25     Level: 061023                GRDOUTI
c               D. Strimaitis
c
c --- ADAPTED FROM:
c --- PRTMET    Version: 4.44      Level: 040923               GRDOUTI
c               F. Robe, D. Strimaitis
c
c --- PURPOSE:  Write out a 'GRD' file compatible with SURFER for
c               making plots ---- ABS(INTEGER) field version
c
c --- INPUTS:
c             IO - integer                - output unit number
c        FILENAM - character*132          - name of output file
c           IVAR - int. array (mxnx,mxny) - 2-D variable to plot
c            FMT - character*40           - format for output record
c            ZLO - real                   - lower limit of range
c            ZHI - real                   - upper limit of range
c                                           (do not use if lo=hi=0)
c            IMG - integer                - image file flag
c                                            0: Image uses standard GRD
c                                               or file used for contour
c                                              (data at cell center)
c                                            1: Image uses shifted range
c                                              (data cover entire cell)
c      NBX     - integer                - X Index of the lower left
c                                         gridpoint of subdomain
c      NBY     - integer                - Y Index of the lower left
c                                         gridpoint of subdomain
c      NEX     - integer                - X Index of the lower right
c                                         gridpoint of subdomain
c      NEY     - integer                - Y Index of the upper left
c                                         gridpoint of subdomain
c      DELX    - real                   - Length of grid cell (km)
c      X0KM    - real                   - X coord. of origin (km)
c      Y0KM    - real                   - Y coord. of origin (km)
c
c --- OUTPUT: output file (unit=io)
c
c --- GRDOUTI called by:  MET1
c --- GRDOUTI calls:      none
c
c---------------------------------------------------------------------------
      include 'params.trj'

      integer ivar(mxnx,mxny)
      character*132 filenam
      character*40 fmt

      data ibig/9999999/

c --- Check range of cells
      if(nbx.GT.nex .OR. nbx.LT.1 .OR. nex.GT.mxnx .OR.
     &   nby.GT.ney .OR. nby.LT.1 .OR. ney.GT.mxny) then
         write(*,*)
         write(*,*)'GRDOUTI: Invalid grid range requested'
         write(*,*)'nbx,nex = ',nbx,nex
         write(*,*)'nby,ney = ',nby,ney
         write(*,*)'Expected nbx<nex, nby<ney within the range'
         write(*,*)'1 to mxnx = ',mxnx,' and 1 to mxny = ',mxny
         stop
      endif

c --- Check image type option
      if(img.LT.0 .OR. img.GT.1) then
         write(*,*)'GRDOUTI: Invalid IMAGE flag requested = ',img
         write(*,*)'Expected 0 or 1'
         stop
      endif

c --- Set X-Y coordinates of the begin/end gridpoints (km)
      xbgkm=x0km+(nbx-0.5)*delx
      ybgkm=y0km+(nby-0.5)*delx
      xegkm=x0km+(nex-0.5)*delx
      yegkm=y0km+(ney-0.5)*delx

c --- Open file
      open(io,file=filenam,status='REPLACE')

c --- Find minimum and maximum value in array:
      imax=-ibig
      imin=ibig
      do j=nby,ney
         do i=nbx,nex
c ---      Impose absolute value of integer variable
           ivar(i,j)=IABS(ivar(i,j))
           if(ivar(i,j).gt.imax)imax=ivar(i,j)
           if(ivar(i,j).lt.imin)imin=ivar(i,j)
         enddo
      enddo
      zmax=FLOAT(imax)
      zmin=FLOAT(imin)

c --- Special case:  drop file for array that is all zero
      if(imax.EQ.0 .AND. imin.EQ.0) then
         close(io, status='DELETE')
         return
      endif

c --- Impose limits from call arguments
      ilo=NINT(zlo)
      ihi=NINT(zhi)
      if(ilo.NE.0 .OR. ihi.NE.0) then
         zmax=zhi
         zmin=zlo
      endif

c --- Do not produce file if there is no spatial gradient
      imin=NINT(zmin)
      imax=NINT(zmax)
      if(imin .EQ. imax) then
         close(io, status='DELETE')
         return
      endif

c --- Shift (km) from cell center to edge for IMAGE plot
      ds=0.5*delx*FLOAT(img)

c --- Header
      write(io,'(a4)')'DSAA'
      write(io,'(2i12)')nex-nbx+1,ney-nby+1
      write(io,'(2f12.4)')xbgkm-ds,xegkm+ds
      write(io,'(2f12.4)')ybgkm-ds,yegkm+ds
      write(io,'(2f12.4)')zmin,zmax

c --- Data records
      do j=nby,ney
         write(io,fmt)(ivar(i,j),i=nbx,nex)
      enddo

      close(io)

      return
      end
c----------------------------------------------------------------------
      subroutine RDHDLOC
c----------------------------------------------------------------------
c
c --- TRAJ2D    Version: 1.25     Level: 081222                RDHDLOC
c               D. Strimaitis
c
c --- PURPOSE:  Read the header records from the TRAJLOC.DAT 
c               trajectory starting location file
c
c --- UPDATES:
c --- V1.1 (061222) to V1.22 (081222)   DGS
c       (1) Allow LL coordinates for location
c
c --- INPUTS:
c
c     Common Block /CTLDAT/ variables:
c            fileloc
c   Parameters:
c            ILST, ILOC
c
c --- OUTPUT:
c
c     Common Block /CTLDAT/ variables:
c            x0,y0,namet,rnlat0t,relon0t,rnlat1t,rnlat2t,
c            iutmznt,feastt,fnortht,pmapt,datumt,datent,utmhemt,
c            clat0t,clon0t,clat1t,clat2t,axbtzt,filetrj
c
c --- RDHDLOC called by: SETUP
c --- RDHDLOC calls:     XTRACTLL, CASE320, NOBLNK320
c----------------------------------------------------------------------
c
      include 'params.trj'
      include 'ctldat.trj'

c --- Local variables
      character*16 clatorig,clonorig
      character*16 dataset,dataver
      character*33 blank33,break33
      character*64 datamod
      character*132 comment1,blank,break
      character*320 char320,blnk320
      logical lutm,llcc,lps,lem,llaza,lttm,lgeo

      data blank33/'                                 '/
      data break33/'.................................'/
      data nlim/1/
      data zero/0.0/

c --- Set blank (132 characters)
      blank(1:33)=blank33
      blank(34:66)=blank33
      blank(67:99)=blank33
      blank(100:132)=blank33

c --- Set break (132 characters)
      break(1:33)=break33
      break(34:66)=break33
      break(67:99)=break33
      break(100:132)=break33

c --- Set blnk320 (320 characters)
      blnk320(1:100)=blank(1:100)
      blnk320(101:200)=blank(1:100)
      blnk320(201:300)=blank(1:100)
      blnk320(301:320)=blank(1:20)

c --- Begin transfer of image to list file
      write(ilst,'(a132)') blank
      write(ilst,'(a132)') break
      write(ilst,'(a132)') blank

c --- Initialize map projection logicals
      lgeo=.FALSE.
      lutm=.FALSE.
      llcc=.FALSE.
      lps=.FALSE.
      lem=.FALSE.
      llaza=.FALSE.
      lttm=.FALSE.

c --- Open the trajectory location file
      open(iloc,file=fileloc, status='OLD')

c --- First record and comments
      read(iloc,'(2a16,a64)') dataset,dataver,datamod
      write(ilst,'(2a16,a64)') dataset,dataver,datamod
      read(iloc,*) ncom
      write(ilst,*) ncom
      do k=1,ncom
         comment1=blank
         read(iloc,'(a132)') comment1
         write(ilst,'(a132)') comment1
      enddo

c --- Map projection
      read(iloc,'(a8)') pmapt
      write(ilst,'(a8)') pmapt
      pmapt=ADJUSTL(pmapt)
c --- Create upper-case version of string
      char320=blnk320
      char320(1:8)=pmapt
      klast=LEN_TRIM(char320)
      call CASE320('UPPER',char320,klast)
      pmapt=char320(1:8)

      if(pmapt.EQ.'UTM     ') then
         lutm=.TRUE.
      elseif(pmapt.EQ.'LCC     ') then
         llcc=.TRUE.
      elseif(pmapt.EQ.'PS      ') then
         lps=.TRUE.
      elseif(pmapt.EQ.'EM      ') then
         lem=.TRUE.
      elseif(pmapt.EQ.'LAZA    ') then
         llaza=.TRUE.
      elseif(pmapt.EQ.'TTM     ') then
         lttm=.TRUE.
      elseif(pmapt.EQ.'LL      ') then
         lgeo=.TRUE.
      else
         write(io6,*)'FATAL Error in RDHDLOC -- Projection'
         write(io6,*)'Expected: ll,utm,lcc,ps,em,laza,ttm'
         write(io6,*)'   Found: ',pmapt
         stop 'Halted in RDHDLOC -- see list file'
      endif

c --- Map projection parameters
      if(LUTM) then
         read(iloc,'(i4,a4)') iutmznt,utmhemt
         write(ilst,'(i4,a4)') iutmznt,utmhemt
      elseif(LLCC) then
         read(iloc,'(4a16)') clat0t,clon0t,clat1t,clat2t
         write(ilst,'(4a16)')  clat0t,clon0t,clat1t,clat2t
      elseif(LPS) then
         read(iloc,'(3a16)') clat0t,clon0t,clat1t
         write(ilst,'(3a16)')  clat0t,clon0t,clat1t
      elseif(LEM.or.LLAZA.or.LTTM) then
         read(iloc,'(2a16)') clat0t,clon0t
         write(ilst,'(2a16)')  clat0t,clon0t
      endif
c --- Map false Easting/Northing
      if(LLCC.or.LLAZA.or.LTTM) then
         read(iloc,*) feastt,fnortht
         write(ilst,*) feastt,fnortht
      else
         feastt=0.0
         fnortht=0.0
      endif

c --- Map DATUM and date-stamp
      read(iloc,'(a8,a12)') datumt,datent
      write(ilst,'(a8,a12)')  datumt,datent
      datumt=ADJUSTL(datumt)
      datent=ADJUSTL(datent)
c --- Create upper-case version of string
      char320=blnk320
      char320(1:8)=datumt
      klast=LEN_TRIM(char320)
      call CASE320('UPPER',char320,klast)
      datumt=char320(1:8)

c --- Map units
      read(iloc,'(a4)') xyunit
      write(ilst,'(a4)')  xyunit
      xyunit=ADJUSTL(xyunit)
c --- Create upper-case version of string
      char320=blnk320
      char320(1:4)=xyunit
      klast=LEN_TRIM(char320)
      call CASE320('UPPER',char320,klast)
      xyunit=char320(1:4)

c --- Time zone
      read(iloc,'(a8)')  axbtzt
      write(ilst,'(a8)') axbtzt
c --- Create upper-case version of string
      char320=blnk320
      char320(1:8)=axbtzt
      klast=LEN_TRIM(char320)
      call CASE320('UPPER',char320,klast)
      axbtzt=char320(1:8)
      if(axbtzt(7:8).NE.'00') then
         write(io6,*)'FATAL Error in RDHDLOC -- Time Zone'
         write(io6,*)'Expected whole hour, Found: ',axbtzt
         stop 'Halted in RDHDLOC -- see list file'
      endif

c --- Location name
      read(iloc,'(a64)') namet
      write(ilst,'(a64)') namet
c --- Location coordinates
      if(LGEO) then
         read(iloc,'(2a16)') clatorig,clonorig
         write(ilst,'(a)') clatorig//clonorig
         call XTRACTLL(io6,'LAT ',clatorig,y0)
         call XTRACTLL(io6,'LON ',clonorig,x0)
      else
         read(iloc,*) x0,y0
         write(ilst,*) x0,y0
      endif
c --- Only convert meters to kilometers for now
      if(xyunit.EQ.'M   ') then
         x0=x0*.001
         x0=x0*.001
      elseif(xyunit.NE.'KM  ' .AND. xyunit.NE.'DEG ') then
         write(io6,*)'FATAL Error in RDHDLOC -- XY Units'
         write(io6,*)'Expected M,KM, or DEG, Found: ',xyunit
         stop 'Halted in RDHDLOC -- see list file'
      endif

c --- Set first part of output filename to deblanked location string
      char320=blnk320
      char320(1:64)=namet
      call NOBLNK320(char320)
      filetrj=char320(1:132)

c --- Process lat/lon character variables to extract content
c --- (dummy values are initialized in block data)
      call XTRACTLL(io6,'LAT ',clat1t,rnlat1t)
      call XTRACTLL(io6,'LAT ',clat2t,rnlat2t)
      call XTRACTLL(io6,'LAT ',clat0t,rnlat0t)
      call XTRACTLL(io6,'LON ',clon0t,relon0t)

c --- End transfer of image to list file
      write(ilst,'(a132)') blank
      write(ilst,'(a132)') break
      write(ilst,'(a132)') blank

      return
      end

c----------------------------------------------------------------------
      subroutine GETLOC(filenam,idone)
c----------------------------------------------------------------------
c
c --- TRAJ2D    Version: 1.25     Level: 081223                 GETLOC
c               D. Strimaitis
c
c --- PURPOSE:  Read the data records from the TRAJLOC.DAT 
c               trajectory starting location & time file
c
c --- UPDATES:
c --- V1.22 (081222) to V1.23 (081223)   DGS
c       (1) Revise time format used for filename to
c           _UTC+HHMM_YYYYMMDDHHSSSS of latest start-time
c
c --- V1.1 (061222) to V1.22 (081222)   DGS
c       (1) Change file extension of output trajectory files to .TRJ
c
c --- INPUTS:
c        FILENAM - character*132     - base name of output file
c
c     Common Block /CTLDAT/ variables:
c                   axbtzt
c
c   Parameters:
c            ILST, ILOC
c
c --- OUTPUT:
c        FILENAM - character*132   - Full name of output file
c          IDONE - integer         - File termination flag
c
c     Common Block /CTLDAT/ variables:
c                   doct,maddconc,
c                   ibyrt,ibmot,ibdyt,ibjdt,ibhrt,ibsect,ibdathrt,
c                   ieyrt,iemot,iedyt,iejdt,iehrt,iesect,iedathrt
c
c --- GETLOC called by: COMP
c --- GETLOC calls:     JULDAY
c----------------------------------------------------------------------
c
      include 'params.trj'
      include 'ctldat.trj'

c --- Local variables
      character*132 filenam, aline
      character*4 aconc

c --- Location time and label record
      do k=1,132
         aline(k:k)=' '
      enddo
      read(iloc,'(a132)',end=999) aline
      write(ilst,'(a132)') aline
      read(aline,*) ibyrt,ibmot,ibdyt,ibhrt,ibsect,
     &              ieyrt,iemot,iedyt,iehrt,iesect,doct

      if(maddconc.GT.0) then
c ---    Pull peak concentration from doct (if present)
         ippbuc=INDEX(doct,'PPB')
         ippblc=INDEX(doct,'ppb')
         ippb=MAX(ippbuc,ippblc)
         if(ippb.GT.0) then
            i1=ippb-4
            i2=ippb-1
            read(doct(i1:i2),'(i)') iconc
            write(aconc(1:3),'(i3.3)') iconc
            aconc(4:4)='_'
            filenam=aconc//filenam
         endif
      endif

c --- Construct date-time variables
      call JULDAY(ilst,ibyrt,ibmot,ibdyt,ibjdt)
      ibdathrt=ibyrt*100000+ibjdt*100+ibhrt
      call JULDAY(ilst,ieyrt,iemot,iedyt,iejdt)
      iedathrt=ieyrt*100000+iejdt*100+iehrt

c --- Add last beginning date-time to base filename
      klast=LEN_TRIM(filenam)
c --- Time zone
      k1=klast+1
      klast=klast+10
      filenam(k1:klast)='_'//axbtzt//'_'
c --- Year-Month-Day-Hour-Second
      k1=klast+1
      klast=klast+14
      write(filenam(k1:klast),'(i4.4,3i2.2,i4.4)') ieyrt,iemot,
     &                                       iedyt,iehrt,iesect
c --- File extension
      filenam=filenam(1:klast)//'.trj'

c --- Set return code
      idone=0
      return
999   idone=1
      return

      end

c----------------------------------------------------------------------
      subroutine tshort(ilst,nsecdt,ndhrb,nsecb,ndhre,nsece)
c----------------------------------------------------------------------
c
c --- TRAJ2D    Version: 1.25     Level: 081222                 TSHORT
c               D. Strimaitis
c
c --- PURPOSE:  Tests year/day/hour and seconds variables at beginning
c               and end of a period against the nominal length of the
c               period and adjusts begin-time to obtain the full
c               length if short.  This allows snapshot data to be
c               assigned a length equal to the interval between
c               snapshots.
c
c --- INPUTS:
c          ILST - integer - List file unit number
c        NSECDT - integer - Number of seconds in a full interval
c         NDHRB - integer - YYYYJJJHH at beginning of period
c         NSECB - integer - SSSS at beginning of period
c         NDHRE - integer - YYYYJJJHH at end of period
c         NSECE - integer - SSSS at end of period
c
c
c --- OUTPUT:
c         NDHRB - integer - Updated YYYYJJJHH at beginning of period
c         NSECB - integer - Updated SSSS at beginning of period
c
c --- TSHORT called by:  RDMET
c --- TSHORT calls:      DELTSEC, DEDAT, INCRS
c----------------------------------------------------------------------

c --- Get interval between begin and end times
      call DELTSEC(ndhrb,nsecb,ndhre,nsece,ndelsec)

      if(ndelsec.LT.nsecdt) then
c ---    Move begin time back to obtain full interval
         ninc=-nsecdt
         ndhrb=ndhre
         nsecb=nsece
         call DEDAT(ndhrb,iyr,ijul,ihr)
         call INCRS(ilst,iyr,ijul,ihr,nsecb,ninc)
         ndhrb=iyr*100000+ijul*100+ihr
      endif

      return
      end
c----------------------------------------------------------------------
      subroutine wrhdtrj
c----------------------------------------------------------------------
c
c --- TRAJ2D    Version: 1.25     Level: 081223                WRHDTRJ
c ---           D. Strimaitis
c
c --- PURPOSE:  Write header to trajectory file
c
c --- UPDATES:
c --- V1.22 (081222) to V1.23 (081223)   DGS
c       (1) Reverse order of columns from Lat,Lon to Lon,Lat
c
c --- INPUTS:
c
c      common/QA/
c            ver,level
c
c      common/CONTROL/xyunit,namet,doct,mcoord,axbtzt,
c            ibyrt,ibmot,ibdyt,ibjdt,ibhrt,ibsect,ibdathrt,
c            ieyrt,iemot,iedyt,iejdt,iehrt,iesect,iedathrt
c
c      common/WINDHD/mode,
c             iutmzn,feast,fnorth,pmap,datum,daten,utmhem,
c             clat0,clon0,clat1,clat2
c
c     Parameters:
c           MXFIL, ITRJ
c
c --- OUTPUT:
c
c
c --- WRHDTRJ called by:  COMP
c --- WRHDTRJ calls:      none
c----------------------------------------------------------------------
      include 'params.trj'
      include 'ctldat.trj'
      include 'qa.trj'
      include 'windhd.trj'

c --- Local variables
      character*16 blnk16,dname,dver
      character*64 doc64
      character*20 times
      character*26 dates
      character*4 month(12)

      data blnk16/'                '/
      data dname/'TRAJ2D.TRJ      '/
      data dver/'1.0             '/
      data times/'0000-0000 (UTC+0000)'/
      data dates/'00, 0000 (Julian Day 000) '/
      data month/'JAN ','FEB','MAR','APR','MAY','JUN',
     &           'JUL ','AUG','SEP','OCT','NOV','DEC'/

c --- Set dataset doc string
      do n=1,4
         i1=(n-1)*16+1
         i2=i1+15
         doc64(i1:i2)=blnk16
      enddo
      doc64(1:21)='Initial Configuration'

c --- Set label strings for header comment block 2
      times(12:19)=axbtzt
      write(times(1:2),'(i2.2)') ibhrt
      write(times(3:4),'(i2.2)') ibsect/60
      write(times(6:7),'(i2.2)') iehrt
      write(times(8:9),'(i2.2)') iesect/60
      write(dates(1:2),'(i2.2)') ibdyt
      write(dates(5:8),'(i4.4)') ibyrt
      write(dates(22:24),'(i3.3)') ibjdt
c --- Screen for single release time
      it1=1
      it2=20
      if(times(1:4).EQ.times(6:9)) it1=6
      klast=LEN_TRIM(namet)


c --- File ID record
      write(itrj,'(2a16,a64)') dname,dver,doc64

c --- Comment block 1
      n1=1
      write(itrj,'(i1)') n1
      write(itrj,'(a)') 'Produced by TRAJ2D  Version: '//ver//
     &                  'Level: '//level

c --- Comment block 2
      n2=3
      write(itrj,'(i1)') n2
      write(itrj,*)'Back-Trajectories Starting from '//namet(1:klast)
      write(itrj,*)'at '//times(it1:it2)//' on '//month(ibmot)//dates
      write(itrj,*) doct

c --- Coordinate documentation (space and time)
      if(mcoord.EQ.0) then
         write(itrj,'(a8)') 'LL      '
         write(itrj,'(a8,a12)') 'WGS-84  ',daten
         write(itrj,'(a4)') ' DEG'
         write(itrj,'(a8)') axbtzt
      elseif(mcoord.EQ.1) then
         write(itrj,'(a8)') pmap
         if(pmap.EQ.'UTM     ') then
            write(itrj,'(i4,a4)') iutmzn,utmhem
         else
            if(pmap.EQ.'LCC     ') then
               write(itrj,'(4a16)') clat0,clon0,clat1,clat2
            elseif(pmap.EQ.'PS      ') then
               write(itrj,'(4a16)') clat0,clon0,clat1
            else
               write(itrj,'(4a16)') clat0,clon0
            endif
            if(pmap.EQ.'LCC     ' .or. 
     &         pmap.EQ.'LAZA    ' .or.
     &         pmap.EQ.'TTM     ') then
               write(itrj,'(2(1x,e14.7))') feast,fnorth
            endif
         endif
         write(itrj,'(a8,a12)') datum,daten
         write(itrj,'(a4)') '  KM'
         write(itrj,'(a8)') axbtzt
      endif

c --- Comment block 3
      n3=3
      write(itrj,'(i1)') n3
      if(mcoord.EQ.0) then
         write(itrj,'(2a36)')'Trajectory Start           Location ',
     &                       '          Time Back    Current Time '
         write(itrj,'(2a36)')' YYYYJJJHH  SSSS     LON(DEG)      L',
     &                       'AT(DEG)     HOURS      JJJ HH  SSSS '
      elseif(mcoord.EQ.1) then
         write(itrj,'(2a34)')'Trajectory Start        Location  ',
     &                       '        Time Back    Current Time '
         write(itrj,'(2a34)')' YYYYJJJHH  SSSS     XKM       YKM',
     &                       '          HOURS      JJJ HH  SSSS '
      endif
      write(itrj,*)

      return
      end
